/* Code for insitalising PlayStation before executing the PocketStation TRC checker
*
*	NOTES   
*
*	CHANGED		PROGRAMMER		REASON
*	-------  	----------		------
*	18/02/99	Mike Kav		Created
*
*/									

#include <sys/types.h>
#include <stdio.h>
#include <strings.h>
#include <kernel.h>
#include <libsn.h>
#include <libgte.h>
#include <libgpu.h>
#include <libetc.h>
#include <libgs.h>

#include "main.h"
#include "control.h"
#include "memcard.h"

#define PAL		1			
#define NTSC	2

DB						db[2];		
short					cdb = 0;			// Current double buffer.
static volatile int		videoMode = PAL;	// Current Video Mode
long					fIdA;				// Applic font id.
static volatile long	frameNo = 0;		// Current frame No.

#ifdef FINAL
u_long _ramsize   = 0x00200000;				// Use 2MB for final.
u_long _stacksize = 0x00004000;
#else
u_long _ramsize   = 0x00800000;				// Use 8MB for development.
u_long _stacksize = 0x00008000;
#endif

int main(void)
{
	InitSys();
	ClearVRAM();
	InitEnvs(db);
	PutDispEnv(&db[cdb].disp);
	SetDispMask(1);

	MemManager();

	CloseSys();
	return 0;
}

/*
*
*       NAME            void InitSys(void)
*
*       FUNCTION        Does the necessary initialization
*
*       NOTES           Copied from MOVIE2 for some reason
*
*
*       CHANGED         PROGRAMMER      REASON
*       -------         ----------      ------
*       18/02/99        Mike Kav        Created
*
*/
void InitSys(void)
{
	ResetGraph(0);
	ResetCallback();
	SetGraphDebug(0);

	if(videoMode == PAL)
		SetVideoMode(MODE_PAL);
	else
		SetVideoMode(MODE_NTSC);

	InitControllers();
	SndInit();
	InitCardSystem();
	VSyncCallback((void (*)()) VSyncCB);
	return;
}

/*
*
*       NAME            void CloseSys(void)
*
*       FUNCTION        Does the necessary initialization
*
*       NOTES           Copied from MOVIE2 for some reason
*
*
*       CHANGED         PROGRAMMER      REASON
*       -------         ----------      ------
*       18/02/99        Mike Kav        Created
*
*/
void CloseSys(void)
{
	VSyncCallback(NULL);
	StopCallback();
	StopControllers();
	CloseCardSystem();
	ResetGraph(3);
	return;
}

/*
*
*       NAME		void InitEnvs(DB *db)
*
*       FUNCTION	Init the drawing and display environments.
*					Also init the application and profiler
*					FntPrint streams. 
*
*		NOTES
*
*       CHANGED		PROGRAMMER      REASON
*       -------		----------      ------
*       18/02/99	Mike Kav        Created
*
*/
void InitEnvs(DB *db)
{

	// Init the display and drawing environments.
	SetDefDrawEnv(&db[0].draw, 0,   0,       FRAME_X, FRAME_Y);
	SetDefDrawEnv(&db[1].draw, 0,   FRAME_Y, FRAME_X, FRAME_Y);

	SetDefDispEnv(&db[0].disp, 0,   FRAME_Y, FRAME_X, FRAME_Y);
	SetDefDispEnv(&db[1].disp, 0,   0,       FRAME_X, FRAME_Y);
	
	setRECT(&db[0].disp.screen, SCREEN_X, SCREEN_Y, 0, FRAME_Y);	
	setRECT(&db[1].disp.screen, SCREEN_X, SCREEN_Y, 0, FRAME_Y);
	setRGB0(&db[0].draw, 0, 0, 50);
	setRGB0(&db[1].draw, 0, 0, 50);
	db[1].draw.isbg = db[0].draw.isbg = 1;

	// Init font environment.
	FntLoad(960, 256);	
	fIdA = FntOpen(16, 16, FRAME_X, FRAME_Y, 0, 1024);			// Applic stream.

	// Kanji Font stream for memory card titles
	KanjiFntOpen (100,180, 400,500, 704,256, 704,450, 0,960);

	SetDumpFnt(fIdA);
	return;
}

/*
*
*       NAME		void VSyncCB(void)
*
*       FUNCTION	VSyncCallback. Check status of controllers
*					every second (50 VSyncs).
*
*       NOTES		
*
*       CHANGED		PROGRAMMER      REASON
*       -------		----------      ------
*       18/02/99	Mike Kav        Created
*
*/
void VSyncCB(void)
{

	frameNo++;

	if(videoMode == NTSC)
	{
		if (!(frameNo % CHECK_CONTROLLERS_NTSC))
			CheckControllers();
	}
	else
	{
		if (!(frameNo % CHECK_CONTROLLERS_PAL))
			CheckControllers();
	}
	return;
}

/*
*
*       NAME		void ClearVRAM(void)
*
*       FUNCTION	Clear entire contents of VRAM.
*
*       NOTES		
*
*       CHANGED		PROGRAMMER      REASON
*       -------		----------      ------
*       18/02/99	Mike Kav        Created
*
*/
void ClearVRAM(void)
{

	RECT	rectTL, rectTR,
			rectBL, rectBR;
	
	setRECT(&rectTL, 0, 0, 512, 256);
	setRECT(&rectTR, 512, 0, 512, 256);
	setRECT(&rectBL, 0, 256, 512, 256);
	setRECT(&rectBR, 512, 256, 512, 256);

	ClearImage(&rectTL, 0, 0, 0);
	ClearImage(&rectTR, 0, 0, 0);
	ClearImage(&rectBL, 0, 0, 0);
	ClearImage(&rectBR, 0, 0, 0);

	DrawSync(0); // Ensure VRAM is cleared before exit.

	return;
}

/*
*
*       NAME		void SndInit(void)
*
*       FUNCTION	Init sound system and volumes.
*
*       NOTES		
*
*       CHANGED		PROGRAMMER      REASON
*       -------		----------      ------
*       18/02/99	Mike Kav        Created
*
*/
void SndInit(void)
{
	// Clear last 100K of SPU RAM (i.e. maximum reverb workarea). 
	SpuInit();
	SpuSetTransStartAddr(421887);
	SpuWrite0(1024 * 100);

	SsInit();

	SsUtReverbOff();
	SsUtSetReverbType(0);
	SsUtSetReverbDepth(0, 0);

	SsSetTickMode(SS_TICKVSYNC);
	SsSetMVol(127, 127);

	SsSetSerialAttr(SS_SERIAL_A, SS_MIX, SS_SON);
	SsSetSerialVol(SS_SERIAL_A, 127, 127);

	SsUtSetReverbType(SS_REV_TYPE_STUDIO_B);
	SsUtReverbOn();
	VSync(75);						//Delay for a while =;-D
	SsUtSetReverbDepth(40, 40);

	return;
}

/*
*
*       NAME		void SndShutDown(void)
*
*       FUNCTION	Shut down the sound system, clear
*					reverb buffer etc
*
*       NOTES		
*
*       CHANGED		PROGRAMMER      REASON
*       -------		----------      ------
*       18/02/99	Mike Kav        Created
*
*/
void SndShutDown(void)
{

    SsUtReverbOff();
    SsUtSetReverbType(0);
    SsUtSetReverbDepth(0,0);

    // Clear last 100K of SPU RAM (i.e. maximum reverb workarea). 
    SpuSetTransStartAddr(421887);
    SpuWrite0(1024 * 100);

	// Wait until SPU RAM cleared.
	VSync(100);

    SsEnd();            

	return;
} 	
