//
// based on shell
//


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "devi_reg.h"
#include "keys.h"
#include "draw.h"

// globals
volatile unsigned char iDone;
extern unsigned short volume;

// play menu sound
extern void MenuSnd (unsigned long);

// prototype for savesecs
extern void vSaveSector (int iSectorNumber, long * plSectorBuffer);

// prototypes
void vFormatNumber (char * strOutput, int iNumber);
void vFormatCard (int iBlocks);
void exitmenu (void);
unsigned char ucComputeChecksum (unsigned char * pucBlock);

void mymain (void)
	{
	int iBlocks = 1;	
	unsigned char ucLastKeys = 0, ucThisKeys = 0;
	char strOutput[40];
	int iState = 0;
	int iYesNo = 0;

	// initialise
	initkeys();

	iDone = 0;
	ucThisKeys = 0xf;

	// main loop
	while (!iDone)
		{
		// debounce key presses
		ucLastKeys = ucThisKeys;  		
		ucThisKeys = keys; //(~ucLastKeys) & keys;

		if (ucThisKeys == ucLastKeys)
			{
			continue;
			}

		// draw screen
		clearRect(0, 0, 32, 32);

		switch (iState)
			{
			case 0: // get number of blocks
				if (iBlocks < 0)
					{
					printv(2, 2, "ABANDON\nFORMAT");
					}
				else
					{
					printv(2, 2, "FORMAT");
					vFormatNumber(strOutput, iBlocks);
					printv(4, 11, strOutput);
					printv(3, 20, "BLOCKS");
					}

				// if enter is pressed, format
				// if up/down pressed, increase/decrease blocks
				if ((ucThisKeys & (KEYUp | KEYRight)) && (iBlocks < 15))
					{
					iBlocks++;
					}

				if ((ucThisKeys & (KEYDown | KEYLeft)) && (iBlocks > -1))
					{
					iBlocks--;
					}

				if (ucThisKeys & KEYEnter)
					{
					if (iBlocks < 0)
						{
						// quit
						iState = 2;
						}
					else
						{
						iState = 1;
						}
					}
				break;
		
			case 1: // ask yes/no
				printv(2, 2, "FORMAT?");
				printv(4, 11, (iYesNo ? "YES" : "NO"));
				
				if (ucThisKeys & (KEYUp | KEYDown | KEYLeft | KEYRight))
					{
					iYesNo = 1 - iYesNo;
					}

				if (ucThisKeys & KEYEnter)
					{
					if (iYesNo)
						{
						iState = 3;
						}
					else
						{
						iState = 0;					
						}
						
					}
				break;
	
			case 2: // quit
				iDone = 1;
				break;

			case 3:	// format
				printv(2, 2, "FORMAT\nNOW!");
				vFormatCard(iBlocks);
				iDone = 1;
				break;

			default:
				// error!
				break;
			}

		// if enter key pressed > 3 secs, display menu
		if (entertime >= 96)
			{
			exitmenu();
			}
		}
	}
	

// defines
#define	MEMORY_BLOCK_SIZE		(8192)
#define	SECTOR_SIZE				(128)
#define	MEMORY_BLOCK0_ADDRESS	(0x08000000)
#define	MEMORY_BLOCK_HEADER_SIZE	(SECTOR_SIZE*4)

// format a number to a string 
void vFormatNumber (char * strOutput, int iNumber)
	{
	char strTemp[30];
	int iNumDigits = 0;
	int iSingleDigit;

	if (iNumber < 0)
		{
		*strOutput++ = '-';
		iNumber = -iNumber;
		}

	// put number in reverse order
	while (iNumber > 9)
		{
		iSingleDigit = iNumber % 10;
		strTemp[iNumDigits++] = '0' + (char)iSingleDigit;
		iNumber /= 10;
		}

	// do last digit
	strTemp[iNumDigits++] = '0' + (char)iNumber;
	
	// reverse the stringified number
	while (--iNumDigits >= 0)
		{
		*strOutput++ = strTemp[iNumDigits];
		}

	// null-terminate the string
	*strOutput = '\0';
	}

// memory card formatter
void vFormatCard (int iBlocks)
	{
	long lBigBuffer[32 * 3];			// big buffer to guarantee MSB alignment
	long * plAlignedBuffer = NULL;		// pointer to useful bit of buffer
	unsigned char * pcBytes = NULL;		// pointer in bytes for setting...
	int iSectorNo;
	int iCurrentBlock ;
	unsigned char ucChecksum = 0;
	const char strFilename[] = "BISLPSP99990";

	// init buffer to be aligned to MSByte of address
	plAlignedBuffer = (long*) ((((long) lBigBuffer) & 0xffffff00) + 256);
	pcBytes = (unsigned char *) plAlignedBuffer;


	// setup those parts of block info which are constant
	memset(pcBytes, 0, SECTOR_SIZE);	// clear to 00

	// 'file size' is in bytes and a LONG value
	plAlignedBuffer[1] = (long)iBlocks * (long)MEMORY_BLOCK_SIZE;
	
	// 'file name'
	strcpy((char *)&pcBytes[10], strFilename);
	
	// 'pda application flag'
	pcBytes[126] = 1;

	// create blocks for application
	for (iCurrentBlock = 1; iCurrentBlock <= iBlocks; iCurrentBlock++)
		{
		// setup block info sector		
		// 'block list information'
		if (iCurrentBlock == 1)
			{
			// header block
			pcBytes[0] = 0x51;
			}
		else if (iCurrentBlock == iBlocks)
			{
			// last block
			pcBytes[0] = 0x53;
			}
		else
			{
			// intermediate block
			pcBytes[0] = 0x52;
			}
		
		// 'next block'
		if (iCurrentBlock == iBlocks)
			{
			// last block, so no next block
			pcBytes[8] = 0xFF;
			pcBytes[9] = 0xFF;				
			}
		else
			{
			// point to next block
			pcBytes[8] = (unsigned char)iCurrentBlock; //(== next block number - 1)
			pcBytes[9] = 0;
			}
		
		// checksum		
		pcBytes[127] = ucComputeChecksum(pcBytes);

		// save sectors 
		vSaveSector(iCurrentBlock, plAlignedBuffer);
		}
		
	// setup blank block info sector
	memset(pcBytes, 0, SECTOR_SIZE);	// clear to 00
	pcBytes[0] = 0xA0;					// first byte is A0
	pcBytes[8] = 0xFF;					// bytes 8-9 are FF
	pcBytes[9] = 0xFF;						
	pcBytes[127] = 0xA0;				// checksum
	ucChecksum = ucComputeChecksum(pcBytes);

	// write rest as blanks
	for (;iCurrentBlock <= 15; iCurrentBlock++)
		{
		// save sectors 
		vSaveSector(iCurrentBlock, plAlignedBuffer);
		}

	// setup alternate info sector
	pcBytes[0] = 0xFF;					// bytes 0-3 are FF
	pcBytes[1] = pcBytes[2] = pcBytes[3] = 0xFF;					
	pcBytes[8] = 0xFF;					// bytes 8-9 are FF
	pcBytes[9] = 0xFF;					
	pcBytes[127] = ucComputeChecksum(pcBytes);	// checksum

	// save sectors 16-35
	for (iSectorNo = 16; iSectorNo < 36; iSectorNo++)
		{
		vSaveSector(iSectorNo, plAlignedBuffer);
		}

	// rest of sectors are all FF (alternate sectors)
	memset(pcBytes, 0xFF, SECTOR_SIZE);	// clear to 00
	ucChecksum = ucComputeChecksum(pcBytes);

	// save sectors 36-63
	for (iSectorNo = 36; iSectorNo < 64; iSectorNo++)
		{
		vSaveSector(iSectorNo, plAlignedBuffer);
		}
	
	// setup Format ID sector
	// done last in case it was corrupted earlier
	memset(pcBytes, 0, SECTOR_SIZE);	// clear to 00
	pcBytes[0] = 0x4D;
	pcBytes[1] = 0x43;					// set first 2 bytes as 'M' 'C'
	pcBytes[127] = ucComputeChecksum(pcBytes);	// checksum

	// save sector 0
	vSaveSector(0, plAlignedBuffer);
	}

//
// ucComputeChecksum - computes checksum for block by XORing bits 1-127
//
unsigned char ucComputeChecksum (unsigned char * pucBlock)
	{
	unsigned char ucChecksum = 0;
	int iCount = 127;
	
	while (iCount-- > 0)
		{
		ucChecksum = ucChecksum ^ (*pucBlock++);
		}
		
	return ucChecksum;
	}

//
// data for 'contine' and 'exit'
//	
const unsigned long continuedata[] = 
	{
	0x00000000, 0x7575dddc, 0x15549544, 0x75549544, 0x15549544, 0x775495dc, 0x00000000
	};

const unsigned long exitdata[] =
	{
	0x00000000, 0x00755c00, 0x00254400, 0x00249c00, 0x00254400, 0x00255c00, 0x00000000
	};	

//
// exitmenu - displays 'continue/exit' screen
//
void exitmenu (void)
	{
	unsigned long scrbuff[32];
	unsigned char count,option;
	unsigned long frequency;

	stopkeys();

	*PIOClearOutputData = 0x20;
	*DACControl = 0x01;

	MenuSnd(0);

	*DACControl = 0x00;
	*PIOSetOutputData = 0x20;
	frequency = *PMFrequency;
	*PMFrequency = 0;
	option = 0;

	for (count = 0; count < 32; count++)
		{
		scrbuff[count] = BackPlane0[count];
		BackPlane0[count] = 0;
		}

	for (count = 0; count < 7; count++)
		{
		BackPlane0[count + 8] = ~continuedata[count];
		BackPlane0[count + 17] = exitdata[count];
		}

	startkeys();

	while(keys&KEYEnter)
		;

	while(!(keys&KEYEnter))
		{
		while(!(keys & (KEYUp | KEYDown | KEYEnter)))
			{}	// wait for user to press a key

		if (((keys & KEYUp) && (option)) || ((keys & KEYDown) && (!option)))
			{
			for (count = 0; count < 7; count++)
				{
				BackPlane0[count + 8] =~ BackPlane0[count + 8];
				BackPlane0[count + 17] =~ BackPlane0[count + 17];
				}

			while((keys&(KEYUp|KEYDown)) == 0x18)
				;
			option =! option;
			}
		}
	iDone = option;
	while(keys & KEYEnter);
	stopkeys();
	*PMFrequency = frequency;
	startkeys();
	}