/* TRC Checker, currently does all the hard stuff regarding setting
*	up and detecting the insertion of the PocketStation. The user
*	can select a file and run the basic TRC checks on the memory
*	card file
*
*	NOTES   
*
*	CHANGED		PROGRAMMER		REASON
*	-------  	----------  	------
*	18/02/99	Mike Kav        Created
*                                       
*/

#include <sys/types.h>
#include <kernel.h>
#include <libsn.h>
#include <libgte.h>
#include <libgpu.h>
#include <libetc.h>
#include <libmcrd.h>
#include <libmcx.h>
#include <ctype.h>

#include "main.h"
#include "control.h"
#include "memcard.h"

extern DB		db[2];		
DB				*cdbuf;				// pointer, current double buffer
extern short	cdb;				// Current double buffer.
extern long		fIdA;				// Applic font id.

// Done up to here

long cardResult = -1;


// card info structure, holds state and number of files
MEMCARDDETAILS cardInfo;

// data to hold file details for a memory card
struct DIRENTRY cardFiles[MAXBLOCK];

// memory card file memory buffer
CARDDATAEX cardData;

int highlight;
int currentFile;
int checkTRC;

short block = INPUT_DELAY*4;

unsigned long card_result;

int Tested111 = 0;
int Passed111 = 0;
int Tested112 = 0;
int Passed112 = 0;
int Tested113 = 0;
int Passed113 = 0;
int Tested114 = 0;
int Passed114 = 0;

/*
*
*	NAME		void MemManager(void)
*
*	FUNCTION	Main control function
*
*	NOTES		
*
*	CHANGED		PROGRAMMER		REASON
*	-------		----------		------
*	18/02/99	Mike Kav		Created
*
*/

void MemManager(void)
{
	block = INPUT_DELAY*4;

	while(!(Pressed(START_KEY) && !block))
	{
		cdb ^= 1;
		cdbuf = ( cdbuf == &db[1] ) ? &db[0] : &db[1];  

		FntPrint(fIdA, "POCKETSTATION TRC CHECKER - VERSION 1.0\n");       
		FntPrint(fIdA, "=======================================\n\n");       
		
		// If we have accepted a PocketStation then display files or
		// test the TRC issues required
		if(cardInfo.state == SET)
		{

			// If we need to test the TRC issues then do so
			if(checkTRC==1)
			{
				TestTRC();
				DisplayMemCardTitle();
			}
			else
			{
				// Display file list for the user to choose apprpriate save
				FntPrint(fIdA, "FILE LIST\n");
				FntPrint(fIdA, "=========\n");
				DrawFileList();
				DisplayMemCardTitle();

				if (!block && Pressed(UP_KEY))
				{
					if(highlight >0)
						highlight--;
					block = INPUT_DELAY;
				}
				else if (!block && Pressed(DOWN_KEY))
				{
					if(highlight <(cardInfo.numFiles-1))
						highlight++;
					block = INPUT_DELAY;
				}
				else if (!block && Pressed(X_KEY))
				{
					checkTRC=1;
					block = INPUT_DELAY;
				}

				if(highlight!=currentFile)
				{
					// When highlight has been changed, load in relevent
					// memory card file header
					currentFile=highlight;
					GetMemCardFile();
				}
			}
		}
		else
			FntPrint(fIdA, "\n\n\tINSERT POCKETSTATION\n");       
			
		CheckCardState();

		if (block > 0)
			block--;

		DrawSync(0);
		FntFlush(-1);
		KanjiFntFlush(-1);
		VSync(0);

		PutDrawEnv(&db[cdb].draw);
		PutDispEnv(&db[cdb].disp);

	}

	// ensure LED is OFF when main loop is exited
	McxSetLED(0,0);

	return;
}

/*
*
*	NAME		void InitCardSystem()
*
*	FUNCTION	Initialises the memory card system
*
*	NOTES		I am currently using libraries 4.4, the library manual states
*				to use 1 as the parameter if using InitPAD, and that it takes
*				up to 70 VSync's to initialise correctly. This function must
*				be called after initialising the pad
*
*
*	CHANGED		PROGRAMMER		REASON
*	-------		----------		------
*	18/02/99	Mike Kav		Created
*
*/
void InitCardSystem()
{
	MemCardInit(1);
	MemCardStart();
	McxStartCom();
	printf("Card System Initialised\n");
	return;
}

/*
*
*	NAME		void CheckCardState(void)
*
*	FUNCTION	Used to continuously poll the memory card to check for events
*
*	NOTES		The MemCardExist() function is only call every 3rd loop, this
*				is because MemCardExist() calls _card_info() which should only be
*				called once per 3 VSyncs()
*
*
*	CHANGED		PROGRAMMER		REASON
*	-------		----------		------
*	18/02/99	Mike Kav		Created
*
*/
void CheckCardState(void)
{
	static int ctr = 0;

	// First check that at least VSync(3) has occurred between calls
	if(ctr==0)
	{
		ctr=3;
		MemCardExist(0);
		MemCardSync(0,0,&card_result);
		// Check value from MemCard Exist, all possible values
		switch(card_result)
		{
			case McErrNone:
			{
				break;
			}
			case McErrCardNotExist:
			{
				if(cardInfo.state == SET)
					cardInfo.state = UNSET;
				break;
			}
			case McErrCardInvalid:
			{
				break;
			}
			case McErrNewCard:
			{
				// Got new card, clear the relevent values and
				// load up the card
				if(cardInfo.state == UNSET)
				{
					InitMemCardValues();
					InitMemCard();
				}
				break;
			}
		}

	}
	else
	{
		ctr--;
	}

	return;
}

/*
*
*	NAME		void CloseCardSystem()
*
*	FUNCTION	Closes the memory card system
*
*	NOTES		
*
*	CHANGED		PROGRAMMER		REASON
*	-------		----------		------
*	18/02/99	Mike Kav		Created
*
*/
void CloseCardSystem()
{
	MemCardStop();
	MemCardEnd();
	printf("Card System Closed\n");
	return;
}

/*
*
*	NAME		void InitMemCardValues(void)
*
*	FUNCTION	Clear memory card values
*
*	NOTES		
*
*	CHANGED		PROGRAMMER		REASON
*	-------		----------		------
*	18/02/99	Mike Kav		Created
*
*/
void InitMemCardValues(void)
{
	// Clear the cardnfo structure and the menu variables

	cardInfo.state			= UNSET;
	cardInfo.numFiles		= 0;
	cardInfo.rawSerialNo	= 0;
	highlight				= 0;
	currentFile				= -1;
	checkTRC				= 0;

	return;
}

/*
*
*	NAME		int InitMemCard(void)
*
*	FUNCTION	Accepts memory card, checks it is a PocketStation and
*				then loads in all files from memory card
*
*	NOTES		
*
*	CHANGED		PROGRAMMER		REASON
*	-------		----------		------
*	18/02/99	Mike Kav		Created
*
*/
void InitMemCard(void)
{
	int cardCheck = 0;

	// Check what the card state is
	//MemCardAccept(chan) where chan = portNo(0x00||0x10) + 0
	if(MemCardAccept(0)==0)
	{
		return;
	}

	// only break if we have a new card to process rest of routine, otherwise
	// just return and wiat for the user to plug in another card
	
	MemCardSync(0,0,&card_result);

	switch(card_result)
	{
		case McErrNewCard:
		{
			break;
		}
		case McErrNone:
		case McErrCardNotExist:
		case McErrCardInvalid:
		case McErrNotFormat:
		default:
		{
			return;
		}
	}

// Memory Card accepted, now check its a PocketStation ...
// The library documentation states:
// An McxErrSuccess result would mean that a PDA was detected, and an McxErrInvalid
// would mean that a Memory Card was detected. However, if the Memory Card
// connection is determined with McxErrInvalid, the operation should be retried to be
// certain.

	while(cardCheck!=2)
	{
		cardCheck++;

		if(McxCardType(0) != 1)
		{
			// failed to register command
			return;
		}
		if( McxSync(0,0,&card_result) == -1)
		{
			// Error in sync
			return;
		}
	
		switch(card_result)
		{
			case McxErrSuccess:
			{
				// PocketStation detected
				cardCheck=2;
				break;
			}
			case McxErrNoCard:
			case McxErrInvalid:
			case McxErrNewCard:
			default:
			{
				if(cardCheck==2)
					return;
			}
		}
	}

	// Get directory list of files. store them away for use later
	switch( MemCardGetDirentry(0,"*",cardFiles,&cardInfo.numFiles,0,MAXBLOCK))
	{
		case McErrNone:
		{
			break;
		}
		case McErrCardNotExist:
		case McErrCardInvalid:
		case McErrNotFormat:
		default:
		{
			return;
		}
	}

	cardInfo.state = SET;

	return;
}

/*
*
*	NAME		void DrawFileList(void)
*
*	FUNCTION	Draws the current list of available files from the memory
*				card.
*
*	NOTES		
*
*	CHANGED		PROGRAMMER		REASON
*	-------		----------		------
*	18/02/99	Mike Kav		Created
*
*/
void DrawFileList(void)
{
	int i;

	for(i=0;i<cardInfo.numFiles;i++)
	{
		if(highlight == i)
			FntPrint(fIdA,"%d) ~c800%s, NUMBER OF BLOCKS USED: %d~c888\n",(i+1),cardFiles[i].name,BLOCKS(cardFiles[i].size));
		else
		{
			FntPrint(fIdA,"%d) %s, NUMBER OF BLOCKS USED: %d\n",(i+1),cardFiles[i].name,BLOCKS(cardFiles[i].size));
		}
	}

	FntPrint(fIdA,"\n\nPRESS X TO CHECK FILE\n");

	return;
}

/*
*
*	NAME		void GetMemCardFile(void)
*
*	FUNCTION	Clears memory card save data and reads in header information
*
*	NOTES		
*
*	CHANGED		PROGRAMMER		REASON
*	-------		----------		------
*	18/02/99	Mike Kav		Created
*
*/
void GetMemCardFile(void)
{
	// Initialise card save values
	memcpy(&cardData,'\0',sizeof(CARDDATAEX));
	Tested111 = 0;
	Passed111 = 0;
	Tested112 = 0;
	Passed112 = 0;
	Tested113 = 0;
	Passed113 = 0;
	Tested114 = 0;
	Passed114 = 0;

// Only bother reading the first 128 bytes for the header information, only reads
// in up to CLUT in header as rest is not needed
	if(MemCardReadFile(0,cardFiles[currentFile].name,(u_long *)&cardData,0,128)==0)
	{
		printf("GetMemCardFile .. failed to read file\n");
		return;
	}

	MemCardSync(0,0,&card_result);
	switch(card_result)
	{
		case McErrNone:
		{
			break;
		}
		case McErrCardNotExist:
		case McErrCardInvalid:
		case McErrNotFormat:
		default:
		{
			return;
			break;
		}
	}

	return;
}

/*
*
*	NAME		void DisplayMemCardTitle(void)
*
*	FUNCTION	Displays Memory Card Title
*
*	NOTES		
*
*	CHANGED		PROGRAMMER		REASON
*	-------		----------		------
*	18/02/99	Mike Kav		Created
*
*/
void DisplayMemCardTitle(void)
{
	KanjiFntPrint("%s\n",cardData.title);
	KanjiFntPrint("~p%s",cardData.title);
}

/*
*
*	NAME		void TestTRC(void)
*
*	FUNCTION	Controls testing of the various TRC issues, also flashed PocketStation
*				LED 'coz it looks good
*
*	NOTES		If the testing starts becoming long, with lots of test cases
*				this should be changed to a loop with a set of function pointers
*				to each TRC test, executing one per loop
*
*	CHANGED		PROGRAMMER		REASON
*	-------		----------		------
*	18/02/99	Mike Kav		Created
*
*/
void TestTRC(void)
{
	static int ctr		= 0;
	static int toggle	= 0;

	if(ctr==0)
	{
		ctr=7;
		McxSetLED(0,(toggle = !toggle));
	}
	else
		ctr--;

	FntPrint(fIdA,"TRC SECTION STATUS    NOTES\n");
	FntPrint(fIdA,"=========== ========= =====\n");

	TRC111();
	TRC112();
	TRC113();
	TRC114();


	FntPrint(fIdA,"\n\nPRESS X TO RETURN TO FILELIST\n");
	
	if (!block && Pressed(X_KEY))
	{
		checkTRC	= 0;
		block		= INPUT_DELAY;
	}

	return;
}

/*
*
*	NAME		void TRC111(void)
*
*	FUNCTION	Test of section 1.1.1
*
*	NOTES		Section 1.1.1 states: The first 12 bytes of the file name
*				consists of the key code plus the product code. The 7th
*				byte which is normally a '-' (0x2D) should be replaced
*				by
*
*	CHANGED		PROGRAMMER		REASON
*	-------		----------		------
*	18/02/99	Mike Kav		Created
*
*/
void TRC111(void)
{
	// check 7th byte of filename, must be 'P' for extended header
	FntPrint(fIdA,"1.1.1       ");

	if(Tested111==0)
	{
		FntPrint(fIdA,"~c088UNCHECKED~c888 ");

		if(cardFiles[currentFile].name[6] == 'P')
		{
			Passed111 = 1;

		}
		else
			Passed111 = -1;
			
		Tested111 = 1;
	}
	else
	{
		if(Passed111 == 1)
			FntPrint(fIdA,"~c080PASSED~c888    ");
		else
			FntPrint(fIdA,"~c800FAILED~c888    ");
	}
	FntPrint(fIdA,"CHECK PRODUCT CODE IS CORRECT (%s)\n",cardFiles[currentFile].name);

	return;
}

/*
*
*	NAME		void TRC112(void)
*
*	FUNCTION	Test of section 1.1.2
*
*	NOTES		Section 1.1.2 states: The pad field of 28 bytes following the save
*				title name in the file header now contains the following details
*					unsigned char	pad[12];					// must be '\0'
*					unsigned short	animIcons;					// number of animation icons
*					unsigned char	fileType[4];				// PocketStation file type, MCX0,CRD0
*					unsigned char	animPages;					// Animation pages
*					unsigned char	numDrivers;					// Number of user device drivers
*					unsigned char	reserved[4];				// reserved 0x00 '\0'
*					unsigned long	startAddr;
*				The fileType must contain either the string 'MCX0' or 'CRD0'
*
*	CHANGED		PROGRAMMER		REASON
*	-------		----------		------
*	18/02/99	Mike Kav		Created
*
*/
void TRC112(void)
{
	int i;

	FntPrint(fIdA,"1.1.2       ");

	if(Tested112==0)
	{
		FntPrint(fIdA,"~c088UNCHECKED~c888 ");

		if( (strncmp(cardData.fileType,"MCX0",4)==0) || (strncmp(cardData.fileType,"CRD0",4)==0) )
		{
			Passed112 = 1;
		}
		else
			Passed112 = -1;
			
		Tested112 = 1;
	}
	else
	{
		if(Passed112 == 1)
			FntPrint(fIdA,"~c080PASSED~c888    ");
		else
			FntPrint(fIdA,"~c800FAILED~c888    ");
	}

	FntPrint(fIdA,"STRING SHOULD BE EITHER MCX0 or CRD0 (");

	for(i=0;i<4;i++)
	{
		FntPrint("%c", (
							(cardData.fileType[i] != 0x7e)
							 &&
							(isprint(cardData.fileType[i]))
						) ? cardData.fileType[i] : ' ');

	}

	FntPrint(fIdA,")\n");
	return;
}

/*
*
*	NAME		void TRC113(void)
*
*	FUNCTION	Test of section 1.1.3
*
*	NOTES		Not yet tested by this program
*
*	CHANGED		PROGRAMMER		REASON
*	-------		----------		------
*	18/02/99	Mike Kav		Created
*
*/
void TRC113(void)
{
	FntPrint(fIdA,"1.1.3       ");

	if(Tested113==0)
	{
		FntPrint(fIdA,"~c088UNCHECKED~c888 ");
/*
		if(1)
		{
			Passed113 = 1;

		}
		else
			Passed113 = -1;
			
		Tested113 = 1;
*/
	}
	else
	{
		if(Passed113 == 1)
			FntPrint(fIdA,"~c080PASSED~c888    ");
		else
			FntPrint(fIdA,"~c800FAILED~c888    ");

	}
	FntPrint(fIdA,"??\n");

	return;
}

/*
*
*	NAME		void TRC114(void)
*
*	FUNCTION	Test of section 1.1.4
*
*	NOTES		Section 1.1.4 states: The ID number for the individual
*				PocketStation is not changed, this function returns the
*				serial number of the PocketStation, obviously the only
*				way to check this is to get the serial number before
*				and after use.
*
*
*	CHANGED		PROGRAMMER		REASON
*	-------		----------		------
*	18/02/99	Mike Kav		Created
*
*/
void TRC114(void)
{
	char serialStr[11];

	FntPrint(fIdA,"1.1.4       ");

	if(Tested114==0)
	{
		FntPrint(fIdA,"~c088UNCHECKED~c888 ");

		if(McxGetSerial(0, &cardInfo.rawSerialNo)!=1)
		{
			printf("Error getting serial number");
			return;
		}
		if( McxSync(0,0,&card_result) == -1)
		{
			printf("Error in sync \n");
			return;
		}	

		switch(card_result)
		{
			case McxErrSuccess:
			{
				Passed114 = 1;
				break;
			}
			case McxErrNoCard:
			case McxErrInvalid:
			default:
			{
				// set fail, just in case
				Passed114 = -1;
				return;
				break;
			}
		}

		Tested114 = 1;
	}
	else
	{
		if(Passed114 == 1)
			FntPrint(fIdA,"~c808CHECKED   ~c888");
		else
			FntPrint(fIdA,"~c800FAILED,    ~c888");
	}

	// because FntPrint() doesn't support %08d properly
	sprintf(serialStr,"%c%08d",GetSerialLetter(&cardInfo),GetSerialNumber(&cardInfo));
	FntPrint(fIdA,"CHECK SERIAL NUMBER AFTER GAMEPLAY (%s)\n",serialStr);
	
	return;
}