;
; Variable-width font print
;

;
; Mark Baker, 16/12/98
;

FANCY EQU 1

;
; Prints a small 6 line font, average width 4 pixels. 
; However, the width of each character is also stored
; so MNQ are bigger but IL,. are smaller.
; This allows more characters to be displayed on each
; line.
;

	GET PERI.INC
	GET MACRO.INC

	IMPORT fontdatav46		; font defined in vfont.s
	EXPORT printv

	CODE16

	AREA LIB_VFONTC, CODE, READONLY, INTERWORK

;---------------------------------------------
; function printv
; CODE16
; prints text using a proportional font
; Letters it can print: 
;	!"#$%&'()*+,-./0123456789:;<=>?@
;	ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]_
;
; It won't handle lowercase letters, newlines or tabs
;
; optionally (if compiled with FANCY EQU 1)
;	* handles newline
;	* automatically puts newline when it is needed	
;	* uses extra registers r8 & more instructions...
;
; Parameters: r0  X (in dots)
;	r1  Y (in dots)
;	r2  starting address of string
; Return value: none
; Registers used: r0-7, (r8), r9
;---------------------------------------------
XP	RN	0
YP	RN	1
fontline	RN	1
fontchar	RN	2
tmp3	RN	2
BPaddr1	RN	3
mask	RN	4
fptr	RN	5
i1	RN	6

tmp2	RN	7
scrnline	RN	7

U_Xstart	RN	8
U_CHaddr	RN	9

printv
	PUSH	{ r3-r7 }
	PUSH	{ lr }

	; r8 and higher regs cannot be saved with PUSH
	; so do separately

	MOV	r3, r9

 IF (FANCY = 1)
	MOV	r4, r8
	PUSH	{ r3-r4 }
	MOV	U_Xstart, XP		; save initial value of X in high-order register
 ELSE
	PUSH	{ r3 }
 ENDIF

	MOV	U_CHaddr, r2		; r2 will be used as tmp3 so save

	LDR	BPaddr1, =BackPlane0    ; get BPaddr
	LSL	YP, #2					; this is the only time YP is used!
	ADD	BPaddr1, YP				; BPaddr1= relevant address in BackPlane

strloop
	MOV	tmp3, U_CHaddr
	F_LDRB	tmp2, tmp3		; macro F_LDRB (byte load from FLASH)
	CMP	tmp2, #0
	BEQ	end			; stop if null character

 IF (FANCY = 1)
	CMP	tmp2, #0x0A		; handle newline
	BNE calcaddr
	MOV XP, U_Xstart	; restore original X value
	ADD BPaddr1, #6*4	; move down a line
	B nextchar
 ENDIF

calcaddr
	LDR	fptr, =fontdatav46	; detect font data address
	SUB	tmp2, #0x20
	LSL	tmp2, #2
	ADD	fptr, fptr, tmp2	; fptr= relevant font data address

	MOV i1, #6
	ADD BPaddr1, #6*4	; (draw from bottom up)
	MOV	mask, #0xf		; create mask

	LDR fontchar, [fptr]	; load character data

 IF (FANCY = 1)
	; work out next XP now
	LSR tmp2, fontchar, #5+(6*4)
	ADD tmp2, XP
	CMP tmp2, #33			; check if X > 33. If so, go to next line
	BLE endNewline

newline
	MOV XP, U_Xstart	; restore original X value
	ADD BPaddr1, #6*4	; move down a line
endNewline
 ENDIF

lineloop
	LSL	mask, XP		; shift mask to clear part of screen
	SUB BPaddr1, #4		; go up a line
	
	LDR scrnline, [BPaddr1]		; load data for current line
	BIC	scrnline, mask			; erase section

	MOV	mask, #0xf			;
	MOV fontline, fontchar  ;
	AND	fontline, mask		; mask this line out of the character data
	LSL fontline, XP		; shift into place on screen
	
	ORR	scrnline, fontline		; OR to screen line
	STR	scrnline, [BPaddr1]		; write data 

	LSR	fontchar, #4	; shift next line's data into place

	SUB	i1, #1		; decrement loop counter
	BNE	lineloop	; loop?

	; next character
	LSR tmp2, fontchar, #5		; shift to get width of character
	ADD	XP, tmp2				; move X coordinate by that width

	; NB mask already = 0xf
	TST fontchar, mask		; is there a 'next address' specified?
	BEQ nextchar

	; draw the bar at the end of some characters - re-use the I!
	MOV tmp2, #'I'		; set 'I' as the character to draw
	B calcaddr			; loop back to address calculation

nextchar
 IF (FANCY <> 1)
	CMP XP, #32		; if not being fancy, stop when we run off
	BGE end			; the end of the screen
 ENDIF

	MOV	tmp2, #1
	ADD	U_CHaddr, tmp2		; next character
	B	strloop
end

 IF (FANCY = 1)
	POP	{ r3-r4 }
	MOV	r8, r4
 ELSE
	POP { r3 }
 ENDIF

	MOV	r9, r3

	POP	{ r3 }
	MOV	lr, r3
	POP	{ r3-r7 }
	BX	lr

    END
