;
; drawlines.s
;

;
; functions to draw lines & boxes
;

;
; modified to draw to OffscreenBuffer, Mark Baker, 9/2/99
; Mark Baker, 18/12/98
;

	GET	PERI.INC
	
	EXPORT VerticalLine
	EXPORT HorizontalLine
	IMPORT OffscreenBuffer

	CODE16
    AREA	LIB_DRAWLINE, CODE, READONLY, INTERWORK

;-----------------------------------------------------------------
; macro ensureLessThan
; CODE16
;
; ensures that Rn <= Rm, swaps if required. 
; Uses Rt as temporary store in swap
;
;-----------------------------------------------------------------
	MACRO
	ensureLessThan $Rn, $Rm, $Rt
	CMP $Rn, $Rm
	BLE %F90
	MOV $Rt, $Rn
	MOV $Rn, $Rm
	MOV $Rm, $Rt
90	
	MEND

;-----------------------------------------------------------------
; function VerticalLine
; CODE16
;
; draw vertical line
;
; Parameters: 
;	r0 - x
;	r1 - y1 (start, inclusive)
;	r2 - y2 (finish, inclusive)
; Return value: none
; Registers used: r0-r5
;-----------------------------------------------------------------
x		RN	0
y1		RN	1
length	RN	1
y2		RN	2
linebit	RN	3
BPaddr	RN	4
scrnline	RN 5
tmp1	RN	5

VerticalLine
	PUSH { r3-r5 }
	PUSH { lr }

	; set up linebit
	MOV linebit, #0x1			; load 0x1
	LSL linebit, x				; shift to get linebit

	ensureLessThan y1, y2, tmp1	; swap y1 and y2 if y1>y2

	;LDR	BPaddr, =OffscreenBuffer		; get initial screen line
	; load offscreen buffer address to BPaddr1
	LDR BPaddr,OFF_BUFF_ADDR
	LDR BPaddr,[BPaddr,#0]
	LSL	tmp1, y1, #2
	ADD	BPaddr, tmp1

	SUB length, y2, y1			; calculate length of line (-1)

	; loop through lines
02
	LDR scrnline, [BPaddr]		; write line (load, ORR, store)
	ORR scrnline, linebit
	STR scrnline, [BPaddr]
	ADD BPaddr, #4				; increment line pointer
	SUB length, #1				; check for end of loop
	BGE %B02

99
	POP	{ r3 }
	MOV	lr, r3
	POP	{ r3-r5 }
	BX	lr


;-----------------------------------------------------------------
; function HorizontalLine
; CODE16
;
; draw horizontal line
;
; Parameters: 
;	r0 - x1 (start, inclusive)
;	r1 - x2(finish, inclusive) 
;	r2 - y 
; Return value: none
; Registers used: r0-r5
;-----------------------------------------------------------------
x1		RN	0
x2		RN	1
length2	RN	1
y		RN	2
linebit	RN	3
BPaddr	RN	4
scrnline2	RN 2

HorizontalLine
	PUSH { r3-r4 }
	PUSH { lr }

	ensureLessThan x1, x2, r3	; swap y1 and y2 if y1>y2

	;LDR	BPaddr, =OffscreenBuffer		; get initial screen line
	; load offscreen buffer address to BPaddr1
	LDR BPaddr,OFF_BUFF_ADDR
	LDR BPaddr,[BPaddr,#0]	
	LSL	y, #2
	ADD	BPaddr, y

	SUB length2, x2, x1			; calculate length of line - 1
	ADD length2, #1				; calc. length

	; get 1s
	MOV linebit, #0x1			; load 0x1
	LSL linebit, length2		; shift to get 2^length
	SUB linebit, #1				; subtract 1 to get all length bits on

	LSL linebit, x1				; shift into position on screen

	LDR scrnline2, [BPaddr]		; write line (load, ORR, store)
	ORR scrnline2, linebit
	STR scrnline2, [BPaddr]

	POP	{ r3 }
	MOV	lr, r3
	POP	{ r3-r4 }
	BX	lr

OFF_BUFF_ADDR
	DCD      |OffscreenBuffer|
        
	END