;
; drawrect.s
;

;
; functions to draw rectangles
;

;
; modified to print to OffscreenBuffer, Mark Baker 9/2/99
; created Mark Baker, 18/12/98
;

	GET	PERI.INC
	
	EXPORT FillRectangle
	EXPORT EorRectangle
	EXPORT ClearRectangle
	IMPORT OffscreenBuffer

	CODE16
    AREA	LIB_DRAWRECT, CODE, READONLY, INTERWORK

;-----------------------------------------------------------------
; macro ensureLessThan
; CODE16
;
; ensures that Rn <= Rm, swaps if required. 
; Uses Rt as temporary store in swap
;
;-----------------------------------------------------------------
	MACRO
	ensureLessThan $Rn, $Rm, $Rt
	CMP $Rn, $Rm
	BLE %F90
	MOV $Rt, $Rn
	MOV $Rn, $Rm
	MOV $Rm, $Rt
90	
	MEND


;-----------------------------------------------------------------
; function FillRectangle
; CODE16
;
; draws a filled rectangle
;
; Parameters: 
;	r0 - x
;	r1 - y
;	r2 - width
;	r3 - height
; Return value: none
; Registers used: r0-r6
;-----------------------------------------------------------------
x		RN	0
y		RN	1
width	RN	2
length	RN	3
linebit	RN	4
BPaddr	RN	5
scrnline	RN 6

FillRectangle
	PUSH { r3-r6 }
	PUSH { lr }

	; get 1s
	MOV linebit, #0x1			; load 0x1
	LSL linebit, width			; shift to get 2^length
	SUB linebit, #1				; subtract 1 to get all length bits on

	LSL linebit, x				; shift into position on screen

	;LDR	BPaddr, =OffscreenBuffer		; get initial screen line
	; load offscreen buffer address to BPaddr1
	LDR BPaddr,OFF_BUFF_ADDR
	LDR BPaddr,[BPaddr,#0]	
	LSL	y, #2
	ADD	BPaddr, y

	B %F01		; jump to end to test for 0 length

	; loop through lines
02
	LDR scrnline, [BPaddr]		; write line (load, ORR, store)
	ORR scrnline, linebit
	STR scrnline, [BPaddr]
	ADD BPaddr, #4				; increment line pointer
01
	SUB length, #1				; check for end of loop
	BGE %B02

99
	POP	{ r3 }
	MOV	lr, r3
	POP	{ r3-r6 }
	BX	lr


;-----------------------------------------------------------------
; function ClearRectangle
; CODE16
;
; clears a rectangular area (performs a BIt Clear)
;
; Parameters: 
;	r0 - x
;	r1 - y
;	r2 - width
;	r3 - height
; Return value: none
; Registers used: r0-r6
;-----------------------------------------------------------------
x		RN	0
y		RN	1
width	RN	2
length	RN	3
linebit	RN	4
BPaddr	RN	5
scrnline	RN 6

ClearRectangle
	PUSH { r3-r6 }
	PUSH { lr }

	; get 1s
	MOV linebit, #0x1			; load 0x1
	LSL linebit, width			; shift to get 2^length
	SUB linebit, #1				; subtract 1 to get all length bits on

	LSL linebit, x				; shift into position on screen

	;LDR	BPaddr, =OffscreenBuffer		; get initial screen line
	; load offscreen buffer address to BPaddr1
	LDR BPaddr,OFF_BUFF_ADDR
	LDR BPaddr,[BPaddr,#0]	
	LSL	y, #2
	ADD	BPaddr, y

	B %F01		; jump to end to test for 0 length

	; loop through lines
02
	LDR scrnline, [BPaddr]		; write line (load, ORR, store)
	BIC scrnline, linebit
	STR scrnline, [BPaddr]
	ADD BPaddr, #4				; increment line pointer
01
	SUB length, #1				; check for end of loop
	BGE %B02

99
	POP	{ r3 }
	MOV	lr, r3
	POP	{ r3-r6 }
	BX	lr


;-----------------------------------------------------------------
; function EorRectangle
; CODE16
;
; draws a filled rectangle by EORing it - good for highlighting text.
;
; Parameters: 
;	r0 - x
;	r1 - y
;	r2 - width
;	r3 - height
; Return value: none
; Registers used: r0-r6
;-----------------------------------------------------------------
x		RN	0
y		RN	1
width	RN	2
length	RN	3
linebit	RN	4
BPaddr	RN	5
scrnline	RN 6

EorRectangle
	PUSH { r3-r6 }
	PUSH { lr }

	; get 1s
	MOV linebit, #0x1			; load 0x1
	LSL linebit, width			; shift to get 2^length
	SUB linebit, #1				; subtract 1 to get all length bits on

	LSL linebit, x				; shift into position on screen

	;LDR	BPaddr, =OffscreenBuffer		; get initial screen line
	; load offscreen buffer address to BPaddr1
	LDR BPaddr,OFF_BUFF_ADDR
	LDR BPaddr,[BPaddr,#0]	
	LSL	y, #2
	ADD	BPaddr, y

	B %F01		; jump to end to test for 0 length

	; loop through lines
02
	LDR scrnline, [BPaddr]		; write line (load, ORR, store)
	EOR scrnline, linebit
	STR scrnline, [BPaddr]
	ADD BPaddr, #4				; increment line pointer
01
	SUB length, #1				; check for end of loop
	BGE %B02

99
	POP	{ r3 }
	MOV	lr, r3
	POP	{ r3-r6 }
	BX	lr

	LSL r0,#0
OFF_BUFF_ADDR
	DCD      |OffscreenBuffer|
        
	END