//
// main.c
//

//
// IR communication example
//

//
// Copy this program to 2 PocketStations/DTL-H4000 kits
//
// Press UP to send or DOWN to receive.
// The first 3 sectors of the program are sent, no checking is performed.
// To quit - Hold ENTER until 'EXITING' is displayed, then release ENTER.
// During send/recieve, pressing ENTER will abandon process.
//

// Mark Baker, SCEE, 17/5/199

#include <stdio.h>
#include <stdlib.h>

#include "pdaregs.h"
#include "pdakernel.h"
#include "keys.h"
#include "draw.h"
#include "flash2ram.h"
#include "strutil.h"
#include "../protocol/protocol.h"


#define WAIT_FOR_KEY_RELEASE(key)	\
	{\
	while (keys & (key))\
		{};\
	}

// allocate an offscreen buffer for drawing
unsigned long aulScreenBuffer[BUFFER_SIZE_LONGS];

// global pointer for drawing commands
unsigned long * OffscreenBuffer;

#define NUM_SECTORS 3

int iWaitForKeyPress (unsigned long * pulPressedKey);


//
// can't use main() due to compiler entry point problems
//
// so we use mymain() instead....
//
void mymain (void)
	{
	// array of sectors sent
	int aiSectorsToUse [NUM_SECTORS] = { 0, 1, 2 };
	int iSectorsReceived = 0;	
	unsigned long ulPressedKey = 0;
	char strTemp[10];

	// setup offscreen buffer
	OffscreenBuffer = aulScreenBuffer;

	// initialise keys
	initkeys();
	
	// user has pressed enter to select this application
	// wait for them to release it
	WAIT_FOR_KEY_RELEASE(KEYEnter);	
	
	PrintFont(0, 0, "TEST IR");
	DrawBuffer();
	
	// get a keypress
	while (iWaitForKeyPress(&ulPressedKey) != 1)
		{
		int iReturn = 10;
		
		// if up was pressed
		if (ulPressedKey & KEYUp)
			{
			// send first 3 sectors
			PrintFont(0, 0, "SEND   ");
			DrawBuffer();
			iReturn = iIrSendSectors(aiSectorsToUse, NUM_SECTORS);
			}
		// if down was pressed
		else if (ulPressedKey & KEYDown)
			{
			// receive sectors
			PrintFont(0, 0, "RECEIVE");
			DrawBuffer();
			iReturn = iIrReceiveSectors(aiSectorsToUse, NUM_SECTORS, &iSectorsReceived);
			
			// print number of sectors received
			FormatNumber(strTemp, iSectorsReceived);
			PrintFont(0, 0, "GOT       ");
			PrintFont(16, 0, strTemp);
			}
			
		// print return value
		switch (iReturn)
			{
			case IR_PROTOCOL_SUCCEED:
				PrintFont(0, 24, "SUCCEED  ");
				break;
				
			case IR_PROTOCOL_FAIL:
				PrintFont(0, 24, "FAIL     ");
				break;
				
			case IR_PROTOCOL_ABANDONED:
				PrintFont(0, 24, "ABANDON  ");
				break;
				
			default:
				break;
			}			
			
		DrawBuffer();
		}

	// quit..
	PrintFont(0, 0, "EXITING");
	DrawBuffer();

	// wait for key release
	startkeys();
	WAIT_FOR_KEY_RELEASE(KEYEnter);	
	stopkeys();
	}
	
	
#define ENTER_TIME 600
	
// wait for keypress
int iWaitForKeyPress (unsigned long * pulPressedKey)
	{
	startkeys();
	
	// await keypress
	
	while (!(keys & (KEYLeft | KEYRight | KEYUp | KEYDown | KEYEnter)))
		{}
	
	*pulPressedKey = keys;
	
	while (keys & (KEYLeft | KEYRight | KEYUp | KEYDown | KEYEnter))
		{
		// wait for unpress or enter held down
		if (entertime > ENTER_TIME)
			{
			// exit!
			return 1;
			}
		}
		
	// change back to original clockspeed
	stopkeys();
	
	return 0;
	}

