//
// drawrect.c
//

// C versions of rectangle drawing functions
// safe to use for negative values of y.
// also can compile to use thumb or arm...

// Mark Baker 26/2/99

#include <stdlib.h>
#include <stdio.h>
#include "draw.h"

#define ULONG(l)	((unsigned long)l)

// FillRectangle() fills a rectangle
void FillRectangle (int x, int y, int width, int height)
	{
	// get line
	unsigned long ulRectLine = ((ULONG(1) << width) - 1) << x;
	
	// draw rectangle
	while ((height-- > 0) && (y < 32))
		{
		if (y >= 0)
			{
			OffscreenBuffer[y] = OffscreenBuffer[y] | ulRectLine;
			}
		y++;
		}
	}


// ClearRectangle() clears a rectangle
void ClearRectangle (int x, int y, int width, int height)
	{
	// get line mask
	unsigned long ulRectLine = !(((ULONG(1) << width) - 1) << x);
	
	// draw rectangle
	while ((height-- > 0) && (y < 32))
		{
		if (y >= 0)
			{
			OffscreenBuffer[y] = OffscreenBuffer[y] & ulRectLine;
			}
		y++;
		}	
	}


// EorRectangle() performs an Exclusive OR on a rectangular area.
// This is useful for highlighting text when combined with other functions
/* example:
	char * string = "TEST";
	PrintFont(0, 0, string);
	EorRectangle(0, 0, FontStrlen(string), FONT_HEIGHT);
*/
void EorRectangle (int x, int y, int width, int height)
	{
	// get line
	unsigned long ulRectLine = ((ULONG(1) << width) - 1) << x;
	
	// draw rectangle
	while ((height-- > 0) && (y < 32))
		{
		if (y >= 0)
			{
			OffscreenBuffer[y] = OffscreenBuffer[y] ^ ulRectLine;
			}
		y++;
		}
	}

