//
// flash2ram.c (fstrcpy, fmemcpy)
//

// copy from flash to RAM

//
// These functions are intended to allow the C library functions
// to be used, as these only work with strings which are in
// RAM.
//

#include "flash2ram.h"

// strcpy but FROM flash memory
// not TO flash memory!
void fstrcpy (char * dest, const char * src)
	{
	do
		{
		*dest = (char)flashReadByte(src);	
		src++;
		}
	while (*dest++ != '\0');
	}
	
	
// strcpy but FROM flash memory
// not TO flash memory!
void fstrncpy (char * dest, const char * src, unsigned int size)
	{
	while (size-- > 0)
		{
		*dest = (char)flashReadByte(src);

		if (*dest != '\0')
			{
			break;
			}

		src++;
		}
	}
	

// strcat from flash to ram	
void fstrcat (char * dest, const char * src)
	{
	// go to end of destination string
	while (*dest != '\0')
		{
		dest++;
		}

	// append the new string	
	fstrcpy(dest, src);
	}

	
// memcpy but FROM flash memory
// not TO flash memory!
void fmemcpy (unsigned char * dest, unsigned char * src, int nbytes)
	{
	while (nbytes-- > 0)
		{
		// read the byte
		*dest = flashReadByte((char *)src);

		// increment pointers
		dest++;
		src++; 
		}
	}
	

// strlen for strings in Flash
unsigned int fstrlen (const char * src)
	{
	char c;
	unsigned int length = 0;
	
	while (1)
		{
		c = (char)flashReadByte(src);	
		
		if (c == '\0')
			{
			break;
			}
			
		src++;
		length++;
		}
	
	return length;
	}


// compares string in SRAM to string in Flash
// returns 0 if they are different,
// returns 1 if they are identical
int iCompareStringToFlash (char * src, const char * target, int nchars)
	{
	char c1, c2;
	
	do
		{
		if (nchars-- <= 0)
			{
			return 1;
			}
		
		c1 = (char)flashReadByte(target);	
		c2 = *src;
		
		src++;
		target++;
		}
	while ((c1 == c2) && (c1 != '\0'));
	
	return (c1 == c2);
	}
	

// copies data in word-blocks 
void vCopyWordsFromFlash (unsigned short * usDestination, const unsigned short * usSource,
	int iNumWords)
	{
	while (iNumWords-- > 0)
		{
		*usDestination++ = *usSource++;
		}
	}
 
	