//
// flash2ram.h (fstrcpy, fmemcpy)
//

// copy from flash to RAM

//
// These functions are intended to allow the C library functions
// to be used, as these only work with strings which are in
// RAM.
//

#ifndef _FLASH2RAM_H
#define _FLASH2RAM_H

// fstrcpy() copies a string from FLASH to RAM
// see strcpy() for a more complete definition
extern void fstrcpy (char * dest, const char * src);

// fstrcpy() copies a string from FLASH to RAM
// see strcpy() for a more complete definition
extern void fstrncpy (char * dest, const char * src, unsigned int size);

// fstrcat() appends a string from FLASH to RAM
// see strcat() for a more complete definition
extern void fstrcat (char * dest, const char * src);

// fmemcpy() copies bytes from FLASH to RAM
extern void fmemcpy (unsigned char * dest, unsigned char * src, int nbytes);

// fstrlen() gets length of string in FLASH
extern unsigned int fstrlen (const char * src);

// compares string in SRAM to string in Flash
// returns 0 if they are different,
// returns 1 if they are identical
extern int iCompareStringToFlash (char * src, const char * target, int nchars);

// copies data in word units from FLASH to RAM
extern void vCopyWordsFromFlash (unsigned short * usDestination, const unsigned short * usSource,
	int iNumWords);
	
// flashReadByte() reads a single byte from FLASH and returns it.
#define flashReadByte(addr)	( (unsigned char) (* ((short *)(addr)) ) )
#endif