//=================================================================
// 				Device registers
//=================================================================

#ifndef __DEVICE_REGISTERS_H
#define __DEVICE_REGISTERS_H

//=================================================================
// Interrupt Controller
//=================================================================
#define INTStatus               ((unsigned long *)0x0a000000)
#define INTRawStatus            ((unsigned long *)0x0a000004)
#define INTEnableSet            ((unsigned long *)0x0a000008)
#define INTEnableReset          ((unsigned long *)0x0a00000c)
#define INTSourceClear          ((unsigned long *)0x0a000010)

	#define INTRawStatus_ofs	0x04
	#define INTEnableSet_ofs	0x08
	#define INTEnableReset_ofs	0x0c
	#define INTSourceClear_ofs	0x10
	#define INTStatus_r20  		0xa0		// << 20 = 0x0a000000

//=================================================================
// LCD Controller
//=================================================================

#define LCDControl              ((unsigned long *)0x0d000000)
#define BackPlane0              ((unsigned long *)0x0d000100)
#define BackPlane1              ((unsigned long *)0x0d000104)
#define BackPlane2              ((unsigned long *)0x0d000108)
#define BackPlane3              ((unsigned long *)0x0d00010c)
#define BackPlane4              ((unsigned long *)0x0d000110)
#define BackPlane5              ((unsigned long *)0x0d000114)
#define BackPlane6              ((unsigned long *)0x0d000118)
#define BackPlane7              ((unsigned long *)0x0d00011c)
#define BackPlane8              ((unsigned long *)0x0d000120)
#define BackPlane9              ((unsigned long *)0x0d000124)
#define BackPlane10             ((unsigned long *)0x0d000128)
#define BackPlane11             ((unsigned long *)0x0d00012c)
#define BackPlane12             ((unsigned long *)0x0d000130)
#define BackPlane13             ((unsigned long *)0x0d000134)
#define BackPlane14             ((unsigned long *)0x0d000138)
#define BackPlane15             ((unsigned long *)0x0d00013c)
#define BackPlane16             ((unsigned long *)0x0d000140)
#define BackPlane17             ((unsigned long *)0x0d000144)
#define BackPlane18             ((unsigned long *)0x0d000148)
#define BackPlane19             ((unsigned long *)0x0d00014c)
#define BackPlane20             ((unsigned long *)0x0d000150)
#define BackPlane21             ((unsigned long *)0x0d000154)
#define BackPlane22             ((unsigned long *)0x0d000158)
#define BackPlane23             ((unsigned long *)0x0d00015c)
#define BackPlane24             ((unsigned long *)0x0d000160)
#define BackPlane25             ((unsigned long *)0x0d000164)
#define BackPlane26             ((unsigned long *)0x0d000168)
#define BackPlane27             ((unsigned long *)0x0d00016c)
#define BackPlane28             ((unsigned long *)0x0d000170)
#define BackPlane29             ((unsigned long *)0x0d000174)
#define BackPlane30             ((unsigned long *)0x0d000178)
#define BackPlane31             ((unsigned long *)0x0d00017c)

	#define BackPlane0_ofs		0x100
	#define LCDControl_r20		0xD0		// << 20 = 0x0D000000
	#define BackPlane0_r8		0x01		// <<  8 = 0x0D000100

//=================================================================
// TIMER
//=================================================================

#define TIMER0Load              ((unsigned long *)0x0a800000)
#define TIMER0Value             ((unsigned long *)0x0a800004)
#define TIMER0Control           ((unsigned long *)0x0a800008)
#define TIMER1Load              ((unsigned long *)0x0a800010)
#define TIMER1Value             ((unsigned long *)0x0a800014)
#define TIMER1Control           ((unsigned long *)0x0a800018)
#define TIMER2Load              ((unsigned long *)0x0a800020)
#define TIMER2Value             ((unsigned long *)0x0a800024)
#define TIMER2Control           ((unsigned long *)0x0a800028)

	#define TIMER0Load_r20		0xa8		// << 20 = 0x0a800000
	#define TIMER0Value_ofs		0x04
	#define TIMER0Control_ofs	0x08

//=================================================================
// RTC
//=================================================================

#define RTCControl              ((unsigned long *)0x0b800000)
#define RTCInc                  ((unsigned long *)0x0b800004)
#define RTCClock                ((unsigned long *)0x0b800008)
#define RTCCalendar             ((unsigned long *)0x0b80000c)

	#define RTCControl_r20		0xb8		// << 20 = 0x0b800000
	#define RTCInc_ofs		0x04
	#define RTCClock_ofs		0x08
	#define RTCCalendar_ofs		0x0c

//=================================================================
// Power Manager
//=================================================================

#define PMFrequency             ((unsigned long *)0x0b000000)
#define PMStandby               ((unsigned long *)0x0b000004)

        #define PMFrequency_r20            0xb0
	#define PMStandby_ofs		0x04

//=================================================================
// Memory Map
//=================================================================

#define REGRemap                        ((unsigned long *)0x06000000)
#define FLASHREMAPStatus                ((unsigned long *)0x06000004)
#define FLASHACTIVEBlocks               ((unsigned long *)0x06000008)
#define FLASHCodeController             ((unsigned long *)0x0600000c)
#define FLASHDataController             ((unsigned long *)0x06000010)

#define FLASHVIRTUALAddr0               ((unsigned long *)0x06000100)
#define FLASHVIRTUALAddr1               ((unsigned long *)0x06000104)
#define FLASHVIRTUALAddr2               ((unsigned long *)0x06000108)
#define FLASHVIRTUALAddr3               ((unsigned long *)0x0600010c)
#define FLASHVIRTUALAddr4               ((unsigned long *)0x06000110)
#define FLASHVIRTUALAddr5               ((unsigned long *)0x06000114)
#define FLASHVIRTUALAddr6               ((unsigned long *)0x06000118)
#define FLASHVIRTUALAddr7               ((unsigned long *)0x0600011c)
#define FLASHVIRTUALAddr8               ((unsigned long *)0x06000120)
#define FLASHVIRTUALAddr9               ((unsigned long *)0x06000124)
#define FLASHVIRTUALAddr10              ((unsigned long *)0x06000128)
#define FLASHVIRTUALAddr11              ((unsigned long *)0x0600012c)
#define FLASHVIRTUALAddr12              ((unsigned long *)0x06000130)
#define FLASHVIRTUALAddr13              ((unsigned long *)0x06000134)
#define FLASHVIRTUALAddr14              ((unsigned long *)0x06000138)
#define FLASHVIRTUALAddr15              ((unsigned long *)0x0600013c)

	#define REGRemap_r20			0x60
	#define FLASHCodeController_ofs		0x0c
	#define FLASHDataController_ofs		0x10

//=================================================================
// PIO, DAC & LVD
//=================================================================

#define PIOControl                      ((unsigned long *)0x0d800000)
#define PIOSetOutputData                ((unsigned long *)0x0d800004)
#define PIOClearOutputData              ((unsigned long *)0x0d800008)
#define PIOReadInputData                ((unsigned long *)0x0d80000c)
#define DACControl                      ((unsigned long *)0x0d800010)
#define DACData                         ((unsigned long *)0x0d800014)
#define LVDControl                      ((unsigned long *)0x0d800020)

	#define PIOControl_r20			0xd8
	#define PIOSetOutputData_ofs		0x04
	#define PIOClearOutputData_ofs		0x08
	#define PIOReadInputData_ofs		0x0c
	#define LVDControl_ofs			0x20
	#define DACControl_ofs			0x10

//=================================================================
// ( IR port )
//=================================================================
#define IFStaticControl                 ((unsigned long *)0x0c800000)
#define IFDynamicControl                ((unsigned long *)0x0c800004)

	#define IFDynamicControl_ofs		0x04

//=================================================================
// BUTTON ASSIGN
//=================================================================

#define BUTTON_ENTER		0x01
#define BUTTON_RIGHT		0x02
#define BUTTON_LEFT		0x04
#define BUTTON_DOWN		0x08
#define BUTTON_UP		0x10
#define BUTTON_NOUSE		0x20

//-----------------------------------------------------------------

#endif