//
// printfont3.c
//

// function PrintFont
// prints text using a proportional font

// rather than compiling with different options (as in printfont2.c)
// this has both simple 'single line' and complicated 'multiple line' versions of printfont
//
//


#include "draw.h"
#include "flash2ram.h"

extern unsigned long fontdatav46[];

#define ULONG(x) ((unsigned long)(x))

//#define FANCY

// bitclear macro - clears bits in 'value' which are set in 'mask'
// used because there's a lovely BIC assembler instruction to do it,
// both tcc/armcc versions included
#ifdef __thumb
#define BITCLEAR(value, mask)	\
	__asm \
		{\
		BIC value, (mask)\
		}
#else
#define BITCLEAR(value, mask)	\
	__asm \
		{\
		BIC value, value, (mask)\
		}
#endif

// function PrintFont
// prints text using a proportional font
// Letters it can print: 
//	!"#$%&'()*+,-./0123456789:;<=>?@
//	ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]_
//
// It won't handle lowercase letters, newlines or tabs
//

void PrintFont (int x, int y, const char * str)
	{
	// go through string until null terminator found
	for (; 1; str++)
		{
		int i;
		unsigned long fontchar;		
		unsigned char currentCharacter;
		int loopY;

		currentCharacter = flashReadByte(str);
		if (currentCharacter == '\0')
			{
			break;
			}
		
		// get the font data
		additionalChar:
		fontchar = fontdatav46[ULONG(currentCharacter) - 0x20];
	
		// setup for drawing all 6 lines
		i = FONT_HEIGHT;
		loopY = y + FONT_HEIGHT;	// draw from bottom-up

		// line loop
		while (i-- > 0)
			{
			unsigned long fontline;
			unsigned long scrnline;

			// go up a line
			loopY--;
			
			if (x >= 0)
				{
				if ((loopY >= 0) && (loopY < 32))
					{
					// load current line on screen,
					// erase masked section
					scrnline = OffscreenBuffer[loopY];
					BITCLEAR(scrnline, ULONG(0xf) << x);	
		
					// get line of font
					fontline = (fontchar & ULONG(0xf)) << x;
		
					// write to screen with OR
					scrnline |= fontline;
		
					// write back to BP
					OffscreenBuffer[loopY] = scrnline;
					}
				}
			else
				{
				if ((loopY >= 0) && (loopY < 32))
					{
					// load current line on screen,
					// erase masked section
					scrnline = OffscreenBuffer[loopY];
					BITCLEAR(scrnline, ULONG(0xf) >> (-x));	
		
					// get line of font
					fontline = (fontchar & ULONG(0xf)) >> (-x);
		
					// write to screen with OR
					scrnline |= fontline;
		
					// write back to BP
					OffscreenBuffer[loopY] = scrnline;
					}
				}
				
			// shift font to get next line
			fontchar >>= 4;
			}
			
		// next character - add width of this character to x coordinate
		x += (int)(fontchar >> 5);

		if (fontchar & ULONG(0xf))
			{
			// additional character to draw
			currentCharacter = 'I';
			goto additionalChar;
			}

		if (x >= 32)
			{
			// stop when we run off edge of screen
			break;
			}
		}
	}

// function PrintFont
// prints text using a proportional font
// Letters it can print: 
//	!"#$%&'()*+,-./0123456789:;<=>?@
//	ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]_
//
// It won't handle lowercase letters, newlines or tabs
//
//	* handles newline
//	* automatically puts newline when it is needed	

int PrintFontMultiline (int x, int y, const char * str)
	{
	// keep track of starting x coordinate
	int xStart = x;
	
	// go through string until null terminator found
	for (; 1; str++)
		{
		int i;
		unsigned long fontchar;		
		unsigned char currentCharacter;
		int loopY;

		currentCharacter = flashReadByte(str);
		if (currentCharacter == '\0')
			{
			break;
			}
		
		// handle newline
		if (currentCharacter == ULONG(0x0A))
			{
			// start on a new line
			x = xStart;
			y += FONT_HEIGHT;
			continue;
			}

		// get the font data
		additionalChar:
		fontchar = fontdatav46[ULONG(currentCharacter) - 0x20];
	
		// get next x now, if its over 32 go to next line
		if ((x + (fontchar >> 29)) > 33)
			{
			x = xStart;
			y += FONT_HEIGHT;
			}

		// setup for drawing all 6 lines
		i = FONT_HEIGHT;
		loopY = y + FONT_HEIGHT; // draw from bottom-up
	
		// line loop
		while (i-- > 0)
			{
			unsigned long fontline;
			unsigned long scrnline;

			// go up a line
			loopY--;
			
			if ((loopY >= 0) && (loopY < 32))
				{
				// load current line on screen,
				// erase masked section
				scrnline = OffscreenBuffer[loopY];
				BITCLEAR(scrnline, ULONG(0xf) << x);	
	
				// get line of font
				fontline = (fontchar & ULONG(0xf)) << x;
	
				// write to screen with OR
				scrnline |= fontline;
	
				// write back to BP
				OffscreenBuffer[loopY] = scrnline;
				}
				
			// shift font to get next line
			fontchar >>= 4;
			}
			
		// next character - add width of this character to x coordinate
		x += (int)(fontchar >> 5);

		if (fontchar & ULONG(0xf))
			{
			// additional character to draw
			currentCharacter = 'I';
			goto additionalChar;
			}
		}
		
	return (y + FONT_HEIGHT);
	}
