
;
; MODIFIED TO NOT USE TIMER1 AT ALL - THIS IS USED BY KEYBOARD READING
;

;
; REWRITTEN TO BE ARM
;

	GET	header\PERI.S
	GET	header\STRUCT.S

	IMPORT	timer_params
	IMPORT	IF_RMfromTCIRQ

	EXPORT	TIMER_Init
	EXPORT	TIMER_Wait
	EXPORT	TIMER_d
	EXPORT	TIMER_Disable
	EXPORT	TIMER_Stop
	EXPORT	TIMER_Reset
	EXPORT	TIMER_Read
;	EXPORT	IRQ_Callback_TC0
	EXPORT	IRQ_Callback_TC1
	EXPORT	FIQ_Callback_TC2
	EXPORT	TIMER_Enable

;=================================================================
	CODE32
        AREA	LIB_TIMER, CODE, READONLY, INTERWORK
;-----------------------------------------------------------------
; function TIMER_Init
; CODE16
; initialize timer (clear buffer, start, enable interrupt)
; Parameters: r0  timer number
;	r1  TimerLoad (16bit)
;       r2  CLKDIV (2bit)
; Return value: none
; Registers used: r4-7
;-----------------------------------------------------------------
	CODE32
TIMER_Init
	STMDB	SP!, { r4-r7, lr }

	MOV	r5, r0			; save r0
	BL	TIMER_Disable		; must be stopped temporarily
	MOV	r0, r5

	LDR	r7, =timer_params
	ADD	r5, r7, #2
	MOV	r6, #0
	STRB	r6, [r5]		; timerX_buf=0
	STRB	r6, [r5, #1]		; timerX_buf_old=0

	MOV	r4, #INTStatus
	MOV	r6, #0x80
	CMP	r0, #0
	BEQ	%F01
	MOV	r6, r6, LSL #1
	CMP	r0, #1
	MOVNE	r6, r6, LSL #5
01
	STR	r6, [r4, #INTEnableSet_ofs]	; enable timer interrupt

	MOV	r4, #TIMER0Load
	MOVS	r0, r0, LSL #4
	ADD	r4, r4, r0
	STR	r1, [r4]
	MOV	r6, #0x04
	ORRS	r6, r6, r2
	STR	r6, [r4, #TIMER0Control_ofs]	; enable timer

	LDMIA	SP!, { r4-r7 }
	LDMIA	SP!, { r3 }
	BX r3


;-----------------------------------------------------------------
; function TIMER_Reset
; CODE32
; start timer
; Parameters: r0  timer number
;	r1  TimerLoad (16bit)
;       r2  CLKDIV (2bit)
; Return value: none
; Registers used: r6-7
;-----------------------------------------------------------------
	CODE32
TIMER_Reset
	STMDB	SP!, { r5-r7, lr }
	MOV	r5, r0
	BL	TIMER_Stop		; must be stopped temporarily
	MOV	r0, r5
	MOV	r7, #TIMER0Load
	ADD	r7, r7, r0, LSL #4
	STR	r1, [r7]
	MOV	r6, #0x04
	ORRS	r6, r6, r2
	STR	r6, [r7, #TIMER0Control_ofs]
	LDMIA	SP!, { r5-r7, pc }

;-----------------------------------------------------------------
; function TIMER_Enable
; CODE32
; enable timer interrupt
; Parameters: r0  timer number
; Return value: none
; Registers used: r6-7
;-----------------------------------------------------------------
	CODE32
TIMER_Enable
	STMDB	SP!, { r6-r7, lr }
	MOV	r7, #INTStatus
	MOV	r6, #0x80
	CMP	r0, #0
	BEQ	%F01
	MOVS	r6, r6, LSL #1
	CMP	r0, #1
	MOVNE	r6, r6, LSL #5
01
	STR	r6, [r7, #INTEnableSet_ofs]	; enable timer interrupt
	LDMIA	SP!, { r6-r7, pc }

;-----------------------------------------------------------------
; function TIMER_Read
; CODE32
; return timer value
; Parameters: r0  timer number
; Return value: r0 value at that moment
; registers used: r7
;-----------------------------------------------------------------
	CODE32
TIMER_Read
	STMDB	SP!, { r7, lr }
	MOV	r7, #TIMER0Load
	ADD	r7, r7, r0, LSL #4
	LDR	r0, [r7, #TIMER0Value_ofs]
	LDMIA	SP!, { r7, pc }

;-----------------------------------------------------------------
; function TIMER_Wait
; CODE32
; wait until timer interrupt takes place
; Parameters: r0 timer number
; Return value:r0 number of times interrupt occurred since last time
; Registers used: r4-7
;-----------------------------------------------------------------
	CODE32
TIMER_Wait
	LDMDB	SP!, { r4-r7, lr }
	LDR	r7, =timer_params
	ADD	r0, r7, r0, LSL #1
	LDRB	r5, [r0]		; timerX_buf
	LDRB	r6, [r0, #1]		; timerX_buf_old
	SUB	r6, r5, r6		; r6 difference
01
	LDRB	r4, [r0]		; timerX_buf
	CMP	r5, r4
	BEQ	%B01
	STRB	r4, [r0, #1]		; old update
	MOV	r0, r6
	LDMIA	SP!, { r4-r7, pc }

;-----------------------------------------------------------------
; function TIMER_d
; CODE32
; returns the number of times the timer interrupt took place
; Parameters: r0 timer number
; Return value:r0 number of times
; Registers used: r5-7
;-----------------------------------------------------------------
	CODE32
TIMER_d
	STMDB	SP!, { r5-r7, lr }
	LDR	r7, =timer_params
	MOV	r0, r0, LSL #1
	ADD	r0, r7, #2
	LDRB	r5, [r0]		; timerX_buf
	LDRB	r6, [r0, #1]		; timerX_buf_old
	STRB	r5, [r0, #1]		; oldXV
	SUB	r0, r5, r6		; r6 
	LDMIA	SP!, { r5-r7, pc }

;-----------------------------------------------------------------
; function TIMER_Disable
; CODE32
; stop timer, disables interrupt
; Parameters: r0  timer number
; Return value: none
; Registers used: r5-r7
;-----------------------------------------------------------------
	CODE32
TIMER_Disable
	STMDB	SP!, { r5-r7, lr }
	MOV	r7, #INTStatus
	MOV	r6, #0x80
	CMP	r0, #0
	BEQ	%F01
	MOV	r6, r6, LSL #1
	CMP	r0, #1
	MOVNE	r6, r6, LSL #5
01
	STR	r6, [r7, #INTEnableReset_ofs]	; disable timer interrupt

	MOV	r7, #TIMER0Load
	ADD	r7, r7, r0, LSL #4
	LDR	r6, [r7, #TIMER0Control_ofs]
	MOV	r5, #0x04
	BIC	r6, r6, r5
	STR	r6, [r7, #TIMER0Control_ofs]	; stop timer
	LDMIA	SP!, { r5-r7, pc }

;-----------------------------------------------------------------
; function TIMER_Stop
; CODE32
;  stop timer
; Parameters: r0  timer number
; Return value: none
; Registers used: r5-r7
;-----------------------------------------------------------------
	CODE32
TIMER_Stop
	STMDB	SP!, { r5-r7, lr }
	MOV	r7, #TIMER0Load
	ADD	r7, r7, r0, LSL #4
	LDR	r6, [r7, #TIMER0Control_ofs]
	MOV	r5, #0x04
	BIC	r6, r6, r5
	STR	r6, [r7, #TIMER0Control_ofs]	; stop timer
	LDMIA	SP!, { r5-r7, pc }

;=================================================================
	CODE32
        AREA	LIB_TIMER32, CODE, READONLY
;-----------------------------------------------------------------
; function IRQ_Callback_TC0
; CODE32
; IRQ callback TIMER0
; Parameters: r0 INTStatus & INTEnableSet
;       r1 address of INTStatus
; Return value: none
; Registers used: 
;-----------------------------------------------------------------
;	CODE32
;IRQ_Callback_TC0
;	STMDB	sp!, {r5-r7, lr}
;	MOV	r6, #0x80
;	MOV	r5, #INTStatus
;	STR	r6, [r5, #INTSourceClear_ofs]	; edge^Ȃ̂ŃNA
;
;	LDR	r7, =timer_params
;	LDRB	r6, timer0_buf
;	ADD	r6, r6, #1
;	STRB	r6, timer0_buf
;
;	BL	IF_RMfromTCIRQ		; IFʐMɏs
;	LDMIA	sp!, {r5-r7, pc}	; ߂

;-----------------------------------------------------------------
; function IRQ_Callback_TC1
; CODE32
; IRQ callback TIMER1
; Parameters: r0 INTStatus & INTEnableSet
;       r1 address of INTStatus
; Return value: none
; Registers used: 
;-----------------------------------------------------------------
	CODE32
IRQ_Callback_TC1
	STMDB	sp!, {r5-r7, lr}
	MOV	r6, #0x00000100
	MOV	r5, #INTStatus
	STR	r6, [r5, #INTSourceClear_ofs]	; edge^Ȃ̂ŃNA

	LDR	r7, =timer_params
	LDRB	r6, timer1_buf
	ADD	r6, r6, #1
	STRB	r6, timer1_buf

	BL	IF_RMfromTCIRQ		; IFʐMɏs	
	LDMIA	sp!, {r5-r7, pc}	; ߂

;-----------------------------------------------------------------
; function FIQ_Callback_TC2
; CODE32
; FIQ callback TIMER2
; Parameters: r0 INTStatus & INTEnableSet
;       r1 address of INTStatus
; Return value: none
; Registers used: r8-r9

;-----------------------------------------------------------------
	CODE32
FIQ_Callback_TC2
	STMDB	sp!, {r7-r9, lr}
	MOV	r8, #0x00002000
	MOV	r9, #INTStatus
	STR	r8, [r9, #INTSourceClear_ofs]	; edge^Ȃ̂ŃNA

	LDR	r7, =timer_params
	LDRB	r8, timer2_buf
	ADD	r8, r8, #1
	STRB	r8, timer2_buf
	LDMIA	sp!, {r7-r9, pc}

;-----------------------------------------------------------------
	END

