//
// low level IR sample code
//

#ifndef _IF_H
#define _IF_H
extern void IF_RMInit (unsigned long ClockFrequency);
//  Initializes IF remote control mode
//  Parameter	r0  clock frequency (5-8) 1, 2, 4, 8 MHz
//  Return value	none
//  Description	Only needs to be called once before IF communications.
//		Does not need to be called more than once even if send
//		and receive operations are alternated. Can also be used
//		to interrupt ongoing communication. Note that the CPU clock
//		frequency will be changed to the value specified by r0.

extern void IF_RMSendStart (unsigned long NumBitsToSend, unsigned char * XmitData, 
	unsigned long RepeatCount, unsigned long FrameLength);
//  Starts IF transmission
//  Parameters	r0  number of bits to send
//		r1  starting address of transmission data
//		r2  repeat count
//		r3  frame length (0: set automatically)
//  Return value	none
//  Description	transmit r0 bits r2 times.
//		Completion is determined using IFGetStatus.
//		If r3 is 0, the frame length is set by calculating a frame period
//		corresponding to the number of bits to send.

extern void IF_RMReceiveStart (unsigned long NumBitsToRecieve, unsigned char ** RecieveBufferTable,
	unsigned long RetrievalCount, unsigned long TimeoutValue);
//  Starts IF reception
//  Parameters	r0  number of bits to receive
//		r1  address for data storage address array (buffer table)
//		r2  data retrieval count (=number of buffers)
//		r3  timeout value for waiting for guide pulse (0: set automatically)
//		    (usually can be set to the same value as the frame length of IF_RMSendStart)
//  Return value	none
//  Description	Receive r0 bits r2 times.
//		If there is no signal, time out based on the value specified
//		in r3. Use IFGetStatus to determine completion.
//		Note that r1 is a buffer table address rather than a direct
//		buffer address.

extern unsigned long IF_RMGetStatus (void);
//  Returns IF status
//  Parameters	none
//  Return value	r0  IF status
//		bit0 1: receiving
//		bit1 1: receive completed normally
//		bit2 1: receive timeout exit
//		bit4 1: transmitting
//		bit5 1: transmission completed
//  Description	Returns current communication status

#define IF_STATUS_RECEIVING 			(1<<0)
#define IF_STATUS_RECEIVE_COMPLETE 		(1<<1)
#define IF_STATUS_RECEIVE_TIMEOUT 		(1<<2)
#define IF_STATUS_TRANSMITTING 			(1<<4)
#define IF_STATUS_TRANSMIT_COMPLETE 	(1<<5)

extern unsigned long IF_RMGetmode (void);
//  Returns the IF mode
//  Parameters	none
//  Return value	r0  IF mode (see end)
//  Description	Returns communication status (IF mode) being used internally
//		by IF functions. Generally not necessary to use.

extern unsigned long IF_RMGettimes (void);
//  Returns the number of times remaining for IF communication
//  Parameters	none
//  Return value	r0: Number of times remaining
//  Description	Returns communication progress status. Generally not necessary to use.

extern void TIMER_Init (int TimerNo, unsigned short TimerLoad, int CLKDIV);
// function 
// initialize timer (clear buffer, start, enable interrupt)
// Parameters: r0  timer number
//	r1  TimerLoad (16bit)
//       r2  CLKDIV (2bit)


#endif