//--------------------------------------------------------------------------
// File: genlib.c
// Author: George Bain
// Date: April 21, 1999
// Updated: April 27, 1999 - DrawSprite()         
// Description: Graphic drawing routines.
// Copyright (C) 1999 Sony Computer Entertainment Europe.,
//           All Rights Reserved.  
//--------------------------------------------------------------------------

//--------------------------------------------------------------------------
// I N C L U D E S
//--------------------------------------------------------------------------

#include <devi_reg.h>
#include "main.h"
#include "genlib.h"

			  
//--------------------------------------------------------------------------
// Function: gClearBackPlane
// Description: Clear the BackPlane0 buffer.
// Parameters: none
// Returns: none
// Notes: N/A
//--------------------------------------------------------------------------

void gClearBackPlane( void )
 {
	register cnt;

	for( cnt =0; cnt <32; cnt++ )
		BackPlane0[cnt] = 0;	

 }// end gClearBackPlane 





//--------------------------------------------------------------------------
// Function: gClearScreen
// Description: Clear the screen buffer.
// Parameters: none 
// Returns: none
// Notes: N/A
//--------------------------------------------------------------------------

void gClearScreen( void )
 {
	register cnt;

	for( cnt =0; cnt <32; cnt++ )
		screen[cnt] = 0;

 }// end gClearScreen 





//--------------------------------------------------------------------------
// Function: gUpdateScreen
// Description: Copy screen buffer to BackPlane0 buffer
// Parameters: none
// Returns: none
// Notes: N/A
//--------------------------------------------------------------------------

void gUpdateScreen( void )
 {
	register cnt;

	for( cnt =0; cnt <32; cnt++ )
		BackPlane0[cnt] = screen[cnt];
	

 }// end gUpdateScreen 





//--------------------------------------------------------------------------
// Function: gPlotPixel
// Description: Plot a single pixel.
// Parameters: u_char x: x location 
//			   u_char y: y location
// Returns: none
// Notes: Assumes x - (0~31) and y - (0~31) ranges
//--------------------------------------------------------------------------

void gPlotPixel( u_char x, u_char y )
 {
	screen[y] |= 1<<x;
	 
 }// end gPlotPixel





//--------------------------------------------------------------------------
// Function: gPlotHorzLine
// Description: Plot a horizontal line.
// Parameters: u_char x: x location 
//			   u_char y: y location
//			   u_char width: width of line
// Returns: none
// Notes: Assumes x - (0~31) and y - (0~31) ranges
//--------------------------------------------------------------------------

	   
void gPlotHorzLine( u_char x, u_char y, u_char width )
 {
 
   	register cnt;

   	for( cnt =0; cnt <width; x++,cnt++ )
	     screen[y] |= 1<<x;

 }// end gPlotHorzLine 





//--------------------------------------------------------------------------
// Function: gPlotVertLine
// Description: Plot a vertical line.
// Parameters: u_char x: x location 
//			   u_char y: y location
//			   u_char height: height of line
// Returns: none
// Notes: Assumes x - (0~31) and y - (0~31) ranges	
//--------------------------------------------------------------------------

void gPlotVertLine( u_char x, u_char y, u_char height )
 { 	
	register cnt;

   	for( cnt =0; cnt <height; cnt++ )
 		 screen[y+cnt] |= 1<<x;
  
 }// end gPlotVertLine




 	
//--------------------------------------------------------------------------
// Function: gPlotRect
// Description: Plot a rectangle.
// Parameters: u_char x0: top left x location 
//			   u_char y0: top left y location
//			   u_char x1: bottom right x location
//			   u_char y1: bottom right y location
// Returns: none
// Notes: Assumes x - (0~31) and y - (0~31) ranges	
//--------------------------------------------------------------------------
 
void gPlotRect( u_char x0, u_char y0, u_char x1, u_char y1 )
 {

	register cnt;
	u_char x_length = (x1-x0)+1;
	
	// draw top horz line
   	for( cnt =0; cnt <x_length; x0++,cnt++ )
	     screen[y0] |= 1<<x0; 
  	   	
	// draw bot horz line
   	for( cnt =0,x0-=(x_length); cnt <x_length; x0++,cnt++ )
	     screen[y1] |= 1<<x0;	 

    // draw left vert line
   	for( cnt =0,x0-=(x_length); cnt <(y1-y0)+1; cnt++ )
 		 screen[y0+cnt] |= 1<<x0;
	  
    // draw right vert line
   	for( cnt =0; cnt <y1-y0; cnt++ )
 		 screen[y0+cnt] |= 1<<x1; 
	  	  
	
 }// end gPlotRect 





//--------------------------------------------------------------------------
// Function: gFillRect
// Description: Plot a solid rectangle.
// Parameters: u_char x0: top left x location 
//			   u_char y0: top left y location
//			   u_char width: width of rectangle
//			   u_char height: height of rectangle
// Returns: none
// Notes: Assumes x - (0~31) and y - (0~31) ranges
//--------------------------------------------------------------------------
  	  
void gFillRect( u_char x0, u_char y0, u_char width, u_char height )
 {	 

	register cnt, cnt2;

    for( cnt =0; cnt <height; cnt++ )
	   {
 		 for( cnt2 =0; cnt2 <width; x0++,cnt2++)
	   	      screen[y0+cnt] |= 1<<x0;

		 x0 -= width;
	   }
	

 }// end gFillRect





//--------------------------------------------------------------------------
// Function: gDrawSprite
// Description: Plots image to screen buffer.
// Parameters: short x: x location
//  		   short y: y location
//			   short h: height of image
//             u_long *addr: address of image array
// Returns: none
// Notes: Does boundry checking 
//--------------------------------------------------------------------------

void gDrawSprite( short x, short y, short h, u_long *addr )
 {
	
	u_long	*plotaddr;
	u_long	workgfx;
	register cnt;

	// get Y position
	plotaddr = &screen[0] + y;

	for( cnt =0; cnt <h; cnt++ )
 	  {
		// bottom Y boundry check
		if( plotaddr > &screen[31] ) 
		    return;

		// Y boundry check 
		if( plotaddr >= &screen[0] )
		  {
			// X boundry check
			if( x >=0 )
			    workgfx = *addr<<x;				
			else
				workgfx = *addr>>-(x);				

			*plotaddr |= workgfx;
		  }

		plotaddr++;
		addr++;

	  }// end for

 }// end DrawSprite



//-------------------------------E O F--------------------------------------