//--------------------------------------------------------------------------
// File: main.c
// Author: George Bain
// Date: April 21, 1999
// Description: A game demo for PocketStation
// Copyright (C) 1999 Sony Computer Entertainment Europe.,
//           All Rights Reserved.  
//--------------------------------------------------------------------------

//--------------------------------------------------------------------------
// I N C L U D E S
//--------------------------------------------------------------------------

#include <devi_reg.h>
#include <keys.h>
#include <menu.h>	
#include "main.h"
#include "genlib.h"

//--------------------------------------------------------------------------
// G L O B A L S
//--------------------------------------------------------------------------

short bspeed_x;
short bspeed_y;
short pspeed_x; 
volatile u_char done;
u_long screen[SCRN_SIZE];
RECT paddle,ball; 
OBJECT block[NUM_BLOCKS]; 

//--------------------------------------------------------------------------
// Function: main
// Description: PocketStation demo
// Parameters: 
// Returns: none
// Notes: N/A
//--------------------------------------------------------------------------
	
int mymain( void )
 {

	register cnt = 0;		
	register delay = DELAY;

	// init all
	done = 0;
	bspeed_x = BSPEED_X;
	bspeed_y = BSPEED_Y;
	pspeed_x = PSPEED_X;

	initkeys();

    gClearBackPlane();
	gClearScreen();

	setRECT(&paddle,10,28,PAD_W,PAD_H);
	setRECT(&ball,16,8,BALL_W,BALL_H);
 	InitBlocks();

	while( keys & KEYEnter );

	while( !done )
		{	
 	  
 	  		// get user input	
			if( keys & KEYRight )
			  {			   				
			    paddle.x+=pspeed_x;
	 		  }
			
			if( keys & KEYLeft )
			  {
			    paddle.x-=pspeed_x;
			   
   			  }
			
			if( keys & KEYUp )
			  {	 			    
   			  }
			
			if( keys & KEYDown )
			  {				    
			  }	 
			 
			// check collision   
			CheckBlocks();
	   		CheckBall(); 		
			CheckPaddle(); 	
			
			// move ball 
		    ball.x += bspeed_x;
			ball.y += bspeed_y;						   		
	   	 
			// draw all
			gFillRect(ball.x,ball.y,ball.w,ball.h);
			gFillRect(paddle.x,paddle.y,paddle.w,paddle.h);	
					
			for( cnt =0; cnt <NUM_BLOCKS; cnt++ )
			   {
				 if( block[cnt].state == BLOCK_ALIVE )
				 gFillRect(block[cnt].rect.x,block[cnt].rect.y,
				          block[cnt].rect.w,block[cnt].rect.h);
			   }


			// update back plane
		   	gUpdateScreen();

			// clear screen buffer
			gClearScreen();		 
			
			// delay
			while( delay-- );
	 				
	 		delay = DELAY;		

			if( entertime >=96 ) 
		    	done = exitmenu();

		}// end while

	return(0);

 }// end main





//--------------------------------------------------------------------------
// Function: InitBlocks
// Description: Set the block grid to the top of the screen display.
// Parameters: none
// Returns: none
// Notes: N/A
//-------------------------------------------------------------------------- 

void InitBlocks( void )
 {

	u_char cnt,cnt2,cnt3=0;
  	
   	// row
   	for( cnt =0; cnt <4; cnt++ )
   	   {
		 // col
		 for( cnt2 =0; cnt2 <8; cnt2++ )
		    { 
			  setRECT(&block[cnt3].rect,(cnt2*4),cnt*2,4,2);
			  block[cnt3].state = BLOCK_ALIVE;
			  cnt3++;
			}	  	
  	   }   


 }// end InitBlocks





//--------------------------------------------------------------------------
// Function: CheckBlocks
// Description: Check if a collision has been made between the ball and any
//              blocks.  Remove block from screen and reverse ball speed.
// Parameters: none
// Returns: none
// Notes: N/A
//--------------------------------------------------------------------------

void CheckBlocks( void )
 {

	u_char cnt;
	
	for( cnt=0; cnt <NUM_BLOCKS; cnt++ )
	   {

	     if( ball.x > (block[cnt].rect.x + block[cnt].rect.w-1) ||
			 block[cnt].rect.x > (ball.x + ball.w-1) ||
	       
			 ball.y > (block[cnt].rect.y + block[cnt].rect.h-1) ||
			 block[cnt].rect.y > (ball.y + ball.h-1) )
		  {
	          // NO HIT
		  }
		else
		  {
	          // HIT
			block[cnt].state = BLOCK_DEAD;
			block[cnt].rect.x = block[cnt].rect.y = 0;
			block[cnt].rect.w = block[cnt].rect.h = 0; 
			bspeed_y = BSPEED_Y;
	      }


	   }// end for
  

 }// CheckBlocks 





//--------------------------------------------------------------------------
// Function: CheckPaddle
// Description: Check collision between paddle and the screen boundry.
// Parameters: none
// Returns: none
// Notes: N/A
//--------------------------------------------------------------------------

void CheckPaddle( void )
 {

    if( (paddle.x < 0) )
      {
	    paddle.x += pspeed_x;
      }
    
    if( ((paddle.x+paddle.w) > 32) )
      {
	    paddle.x -= pspeed_x;
      }    
       	  
 }// end CheckPaddle  





//--------------------------------------------------------------------------
// Function: CheckBall
// Description: Check if collision has been made between ball and paddle.
//              All check collision between ball and screen boundry.
// Parameters: none 
// Returns: none
// Notes: N/A
//--------------------------------------------------------------------------

void CheckBall( void )
 {
   
   
    if( ball.x <=0 )
      {
		bspeed_x = BSPEED_X;
      }
         
   	if( ball.x > (31-ball.w) )
      {
	    
		bspeed_x = -BSPEED_X;
	  }  
	   
	
   	if( ball.y <= 0 )
      {
	    
		bspeed_y = BSPEED_Y;
	   
      }  
   	
   	if( ball.y > (31-ball.h) )
      {
	    
		bspeed_y = -BSPEED_Y;
	   
      } 
	 
    if( (ball.y >= (paddle.y-paddle.h)) &&
	    (ball.x >= paddle.x) &&
		(ball.x <= (paddle.x+paddle.w))
      )
	  {
	    bspeed_y = -BSPEED_Y;
		
	  }
  
 }// end CheckBall


//-------------------------------E O F--------------------------------------