//
// printfont2.c
//

// function PrintFont
// prints text using a proportional font

#include "draw.h"
#include "flash2ram.h"

extern unsigned long fontdatav46[];

#define ULONG(x) ((unsigned long)(x))

#define FANCY

// bitclear macro - clears bits in 'value' which are set in 'mask'
// used because there's a lovely BIC assembler instruction to do it,
// both tcc/armcc versions included
#ifdef __thumb
#define BITCLEAR(value, mask)	\
	__asm \
		{\
		BIC value, (mask)\
		}
#else
#define BITCLEAR(value, mask)	\
	__asm \
		{\
		BIC value, value, (mask)\
		}
#endif

// function PrintFont
// prints text using a proportional font
// Letters it can print: 
//	!"#$%&'()*+,-./0123456789:;<=>?@
//	ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]_
//
// It won't handle lowercase letters, newlines or tabs
//
// optionally (if compiled with FANCY)
//	* handles newline
//	* automatically puts newline when it is needed	

void PrintFont (int x, int y, char * str)
	{
	unsigned long * bpaddr;

	#ifdef FANCY
	// keep track of strarting x coordinate
	int xStart = x;
	#endif
	
	bpaddr = OffscreenBuffer + y;
	
	// go through string until null terminator found
	for (; 1; str++)
		{
		int i;
		unsigned long fontchar;		
		unsigned char currentCharacter;

		currentCharacter = flashReadByte(str);
		if (currentCharacter == '\0')
			{
			break;
			}
		
		#ifdef FANCY
		// handle newline
		if (currentCharacter == ULONG(0x0A))
			{
			// start on a new line
			x = xStart;
			bpaddr += 6;  // equivalent to y++;
			continue;
			}
		#endif

		// get the font data
		additionalChar:
		fontchar = fontdatav46[ULONG(currentCharacter) - 0x20];
	
		// setup for drawing all 6 lines
		i = 6;
		bpaddr += 6;	// draw from bottom-up

		#ifdef FANCY
		// get next x now, if its over 32 go to next line
		if ((x + (fontchar >> 29)) > 32)
			{
			x = xStart;
			bpaddr += 6;
			}
		#endif

		// line loop
		while (i-- > 0)
			{
			unsigned long fontline;
			unsigned long scrnline;

			// go up a line
			bpaddr--;
			
			// load current line on screen,
			// erase masked section
			scrnline = *bpaddr;
			BITCLEAR(scrnline, ULONG(0xf) << x);	

			// get line of font
			fontline = (fontchar & ULONG(0xf)) << x;

			// write to screen with OR
			scrnline |= fontline;

			// write back to BP
			*bpaddr = scrnline;
			
			// shift font to get next line
			fontchar >>= 4;
			}
			
		// next character - add width of this character to x coordinate
		x += (int)(fontchar >> 5);

		if (fontchar & ULONG(0xf))
			{
			// additional character to draw
			currentCharacter = 'I';
			goto additionalChar;
			}

		#ifndef FANCY
		if (x >= 32)
			{
			// stop when we run off edge of screen
			break;
			}
		#endif
		}
	}
