//
// protocol.h
//

//
// An example IR high-level protocol that sends and receives 128 byte sectors.
// This is designed to transfer large blocks of memory from flash, and can be used to swap
// memory card save files, etc.
//

// For more documentation see protocol.c

// Mark Baker, SCEE, 17/5/1999


#ifndef _PROTOCOL_H
#define _PROTOCOL_H

// return value from IR protocol
enum
	{
	IR_PROTOCOL_SUCCEED = 0,
	IR_PROTOCOL_FAIL = 1,
	IR_PROTOCOL_ABANDONED
	};


// sector buffer used by IR protocol
// for simplicity this is statically allocated as being 0x200 - 0x280
// you must change where the ARM linker places variables to start at 0x280 for this to work.
#define aucSectorBuffer	((unsigned char *)0x200)


// iIrSendSectors()
// Sends a list of sectors over IR.
// Supply a list of sector numbers to send & number of sectors
//  -- these are 'relative' sector numbers, with 0 being the first sector in the program's blocks.
//     63 being the last sector of the first block, etc..
//     e.g. They must be suitable use use with the kernel 'Write Flash Relative' function
//
// Returns IR_PROTOCOL_FAIL or IR_PROTOCOL_SUCCEED

extern int iIrSendSectors (int * aiSectorsToSend, int iNumberOfSectorsToSend);


// iIrReceiveSectors()
// Receives a list of sectors over IR
// Supply a list of sector number to write to & a maximum number of sectors to receieve.
// The number of sectors actually received will be written to *piNumberOfSectorsReceived
//
// Returns IR_PROTOCOL_FAIL, IR_PROTOCOL_SUCCEED or IR_PROTOCOL_ABANDONED
// Stops if user presses enter (returns IR_PROTOCOL_ABANDONED)

extern int iIrReceiveSectors (int * aiSectorsToRecieve, int iMaximumNumberOfSectorsToReceive, 
	int * piNumberOfSectorsReceived);

#endif