//
// scroll.c
//

//
// scroll from the current offscreen buffer to a new one.
//

#include "draw.h"
#include "pdaregs.h"

//
// scroll the screen from current OffscreenBuffer buffer to new buffer
// NB NewScreen != OffscreenBuffer, must be a different buffer
//
void ScrollScreen (unsigned char direction, long delay, unsigned long * NewScreen)
	{
	int step;
	
	for (step = 0; step < 32; step++)
		{
		long count;

		switch (direction)
			{
			case DIRECTION_DOWN:
			default:
				// shift current buffer down
				for (count = 31; count > 0; count--)
					{
					OffscreenBuffer[count] = OffscreenBuffer[count - 1];
					}
				
				// draw a line from the new buffer at the top
				OffscreenBuffer[0] = NewScreen[31 - step];
				break;
				
			case DIRECTION_UP: 
				// shift current buffer up
				for (count = 0; count < 31; count++)
					{
					OffscreenBuffer[count] = OffscreenBuffer[count + 1];
					}
				
				// draw a line from the new buffer at the top
				OffscreenBuffer[31] = NewScreen[step];
				break;
			
			case DIRECTION_RIGHT: 
				// each line is the same
				for (count = 0; count < 32; count++)
					{
					// shift each line left,
					// or Nth bit from new screen into lowest position
					OffscreenBuffer[count] = (OffscreenBuffer[count] << 1) |
						( (NewScreen[count] >> (31 - step)) & 0x1);
					}
				break;			
			
			case DIRECTION_LEFT:
				// each line is the same
				for (count = 0; count < 32; count++)
					{
					// shift each line left,
					// or 31 - Nth bit from new screen into highest position
					OffscreenBuffer[count] = (OffscreenBuffer[count] >> 1) |
						( (NewScreen[count] >> step) << 31);
					}
				break;
			}	
		
		// draw the screen
		DrawBuffer();
		
		// busy-wait loop for delay
		for (count = 0; count < delay; count++)
			{}
		}
	}
	